@@ -89,23 +89,32 @@ module ExecutionEngine: sig
       module provider [mp] if successful. Creates a JIT if possible, else falls
       back to an interpreter. Raises [Error msg] if an error occurrs. The
       execution engine is not garbage collected and must be destroyed with
-      [dispose ee]. See the function [llvm::ExecutionEngine::create]. *)
+      [dispose ee]. See the function [llvm::EngineBuilder::create]. *)
   val create: Llvm.llmoduleprovider -> t
   
   (** [create_interpreter mp] creates a new interpreter, taking ownership of the
       module provider [mp] if successful. Raises [Error msg] if an error
       occurrs. The execution engine is not garbage collected and must be
       destroyed with [dispose ee].
-      See the function [llvm::ExecutionEngine::create]. *)
+      See the function [llvm::EngineBuilder::create]. *)
   val create_interpreter: Llvm.llmoduleprovider -> t
   
   (** [create_jit mp] creates a new JIT (just-in-time compiler), taking
-      ownership of the module provider [mp] if successful. Raises [Error msg] if
-      an error occurrs. The execution engine is not garbage collected and must
-      be destroyed with [dispose ee].
-      See the function [llvm::ExecutionEngine::create]. *)
+      ownership of the module provider [mp] if successful. This function creates
+      a JIT which favors code quality over compilation speed. Raises [Error msg]
+      if an error occurrs. The execution engine is not garbage collected and
+      must be destroyed with [dispose ee].
+      See the function [llvm::EngineBuilder::create]. *)
   val create_jit: Llvm.llmoduleprovider -> t
   
+  (** [create_fast_jit mp] creates a new JIT (just-in-time compiler) which
+      favors compilation speed over code quality. It takes ownership of the
+      module provider [mp] if successful. Raises [Error msg] if an error
+      occurrs. The execution engine is not garbage collected and must be
+      destroyed with [dispose ee].
+      See the function [llvm::EngineBuilder::create]. *)
+  val create_fast_jit: Llvm.llmoduleprovider -> t
+  
   (** [dispose ee] releases the memory used by the execution engine and must be
       invoked to avoid memory leaks. *)
   val dispose: t -> unit
@@ -152,3 +161,6 @@ module ExecutionEngine: sig
       [ee]. *)
   val target_data: t -> Llvm_target.TargetData.t
 end
+
+external initialize_native_target : unit -> bool
+                                  = "llvm_initialize_native_target"
