@@ -9,7 +9,7 @@
 //
 // This file defines a hash set that can be used to remove duplication of nodes
 // in a graph.  This code was originally created by Chris Lattner for use with
-// SelectionDAGCSEMap, but was isolated to provide use across the llvm code set. 
+// SelectionDAGCSEMap, but was isolated to provide use across the llvm code set.
 //
 //===----------------------------------------------------------------------===//
 
@@ -19,6 +19,7 @@
 #include "llvm/Support/DataTypes.h"
 #include "llvm/ADT/SmallVector.h"
 #include <string>
+#include <iterator>
 
 namespace llvm {
   class APFloat;
@@ -29,7 +30,7 @@ namespace llvm {
 ///      instance of the node in the set.  If the node already exists, return
 ///      it, otherwise return the bucket it should be inserted into.
 ///   2. Given a node that has already been created, remove it from the set.
-/// 
+///
 /// This class is implemented as a single-link chained hash table, where the
 /// "buckets" are actually the nodes themselves (the next pointer is in the
 /// node).  The last node points back to the bucket to simplify node removal.
@@ -37,7 +38,7 @@ namespace llvm {
 /// Any node that is to be included in the folding set must be a subclass of
 /// FoldingSetNode.  The node class must also define a Profile method used to
 /// establish the unique bits of data for the node.  The Profile method is
-/// passed a FoldingSetNodeID object which is used to gather the bits.  Just 
+/// passed a FoldingSetNodeID object which is used to gather the bits.  Just
 /// call one of the Add* functions defined in the FoldingSetImpl::NodeID class.
 /// NOTE: That the folding set does not own the nodes and it is the
 /// responsibility of the user to dispose of the nodes.
@@ -50,7 +51,7 @@ namespace llvm {
 ///    public:
 ///      MyNode(const char *N, unsigned V) : Name(N), Value(V) {}
 ///       ...
-///      void Profile(FoldingSetNodeID &ID) {
+///      void Profile(FoldingSetNodeID &ID) const {
 ///        ID.AddString(Name);
 ///        ID.AddInteger(Value);
 ///       }
@@ -62,7 +63,7 @@ namespace llvm {
 /// Eg.
 ///    FoldingSet<MyNode> MyFoldingSet;
 ///
-/// Four public methods are available to manipulate the folding set; 
+/// Four public methods are available to manipulate the folding set;
 ///
 /// 1) If you have an existing node that you want add to the set but unsure
 /// that the node might already exist then call;
@@ -97,34 +98,34 @@ namespace llvm {
 ///    bool WasRemoved = RemoveNode(N);
 ///
 /// The result indicates whether the node existed in the folding set.
-  
+
 class FoldingSetNodeID;
-  
+
 //===----------------------------------------------------------------------===//
 /// FoldingSetImpl - Implements the folding set functionality.  The main
 /// structure is an array of buckets.  Each bucket is indexed by the hash of
 /// the nodes it contains.  The bucket itself points to the nodes contained
 /// in the bucket via a singly linked list.  The last node in the list points
 /// back to the bucket to facilitate node removal.
-/// 
+///
 class FoldingSetImpl {
 protected:
   /// Buckets - Array of bucket chains.
   ///
   void **Buckets;
-  
+
   /// NumBuckets - Length of the Buckets array.  Always a power of 2.
   ///
   unsigned NumBuckets;
-  
+
   /// NumNodes - Number of nodes in the folding set. Growth occurs when NumNodes
   /// is greater than twice the number of buckets.
   unsigned NumNodes;
-  
+
 public:
   explicit FoldingSetImpl(unsigned Log2InitSize = 6);
   virtual ~FoldingSetImpl();
-  
+
   //===--------------------------------------------------------------------===//
   /// Node - This class is used to maintain the singly linked bucket list in
   /// a folding set.
@@ -133,44 +134,50 @@ public:
   private:
     // NextInFoldingSetBucket - next link in the bucket list.
     void *NextInFoldingSetBucket;
-    
+
   public:
 
     Node() : NextInFoldingSetBucket(0) {}
-    
+
     // Accessors
     void *getNextInBucket() const { return NextInFoldingSetBucket; }
     void SetNextInBucket(void *N) { NextInFoldingSetBucket = N; }
   };
 
+  /// clear - Remove all nodes from the folding set.
+  void clear();
+
   /// RemoveNode - Remove a node from the folding set, returning true if one
   /// was removed or false if the node was not in the folding set.
   bool RemoveNode(Node *N);
-  
+
   /// GetOrInsertNode - If there is an existing simple Node exactly
   /// equal to the specified node, return it.  Otherwise, insert 'N' and return
   /// it instead.
   Node *GetOrInsertNode(Node *N);
-  
+
   /// FindNodeOrInsertPos - Look up the node specified by ID.  If it exists,
   /// return it.  If not, return the insertion token that will make insertion
   /// faster.
   Node *FindNodeOrInsertPos(const FoldingSetNodeID &ID, void *&InsertPos);
-  
+
   /// InsertNode - Insert the specified node into the folding set, knowing that
-  /// it is not already in the folding set.  InsertPos must be obtained from 
+  /// it is not already in the folding set.  InsertPos must be obtained from
   /// FindNodeOrInsertPos.
   void InsertNode(Node *N, void *InsertPos);
-  
+
   /// size - Returns the number of nodes in the folding set.
   unsigned size() const { return NumNodes; }
-  
+
+  /// empty - Returns true if there are no nodes in the folding set.
+  bool empty() const { return NumNodes == 0; }
+
 private:
 
   /// GrowHashTable - Double the size of the hash table and rehash everything.
   ///
   void GrowHashTable();
-  
+
 protected:
 
   /// GetNodeProfile - Instantiations of the FoldingSet template implement
@@ -190,61 +197,64 @@ template<typename T> struct FoldingSetTrait {
   static inline void Profile(const T& X, FoldingSetNodeID& ID) { X.Profile(ID);}
   static inline void Profile(T& X, FoldingSetNodeID& ID) { X.Profile(ID); }
 };
-  
+
 //===--------------------------------------------------------------------===//
 /// FoldingSetNodeID - This class is used to gather all the unique data bits of
 /// a node.  When all the bits are gathered this class is used to produce a
-/// hash value for the node.  
+/// hash value for the node.
 ///
 class FoldingSetNodeID {
   /// Bits - Vector of all the data bits that make the node unique.
   /// Use a SmallVector to avoid a heap allocation in the common case.
   SmallVector<unsigned, 32> Bits;
-  
+
 public:
   FoldingSetNodeID() {}
-  
+
   /// getRawData - Return the ith entry in the Bits data.
   ///
   unsigned getRawData(unsigned i) const {
     return Bits[i];
   }
-  
+
   /// Add* - Add various data types to Bit data.
   ///
   void AddPointer(const void *Ptr);
   void AddInteger(signed I);
   void AddInteger(unsigned I);
-  void AddInteger(int64_t I);
-  void AddInteger(uint64_t I);
-  void AddFloat(float F);
-  void AddDouble(double D);
+  void AddInteger(long I);
+  void AddInteger(unsigned long I);
+  void AddInteger(long long I);
+  void AddInteger(unsigned long long I);
+  void AddBoolean(bool B) { AddInteger(B ? 1U : 0U); }
+  void AddString(const char* String, const char* End);
   void AddString(const std::string &String);
-  
+  void AddString(const char* String);
+
   template <typename T>
   inline void Add(const T& x) { FoldingSetTrait<T>::Profile(x, *this); }
-  
+
   /// clear - Clear the accumulated profile, allowing this FoldingSetNodeID
   ///  object to be used to compute a new profile.
   inline void clear() { Bits.clear(); }
-  
+
   /// ComputeHash - Compute a strong hash value for this FoldingSetNodeID, used
   ///  to lookup the node in the FoldingSetImpl.
   unsigned ComputeHash() const;
-  
+
   /// operator== - Used to compare two nodes to each other.
   ///
   bool operator==(const FoldingSetNodeID &RHS) const;
-};  
+};
 
 // Convenience type to hide the implementation of the folding set.
 typedef FoldingSetImpl::Node FoldingSetNode;
 template<class T> class FoldingSetIterator;
 template<class T> class FoldingSetBucketIterator;
-  
+
 //===----------------------------------------------------------------------===//
 /// FoldingSet - This template class is used to instantiate a specialized
-/// implementation of the folding set to the node class T.  T must be a 
+/// implementation of the folding set to the node class T.  T must be a
 /// subclass of FoldingSetNode and implement a Profile function.
 ///
 template<class T> class FoldingSet : public FoldingSetImpl {
@@ -255,12 +265,12 @@ private:
     T *TN = static_cast<T *>(N);
     FoldingSetTrait<T>::Profile(*TN,ID);
   }
-  
+
 public:
   explicit FoldingSet(unsigned Log2InitSize = 6)
   : FoldingSetImpl(Log2InitSize)
   {}
-  
+
   typedef FoldingSetIterator<T> iterator;
   iterator begin() { return iterator(Buckets); }
   iterator end() { return iterator(Buckets+NumBuckets); }
@@ -269,23 +279,23 @@ public:
   const_iterator begin() const { return const_iterator(Buckets); }
   const_iterator end() const { return const_iterator(Buckets+NumBuckets); }
 
-  typedef FoldingSetBucketIterator<T> bucket_iterator;  
+  typedef FoldingSetBucketIterator<T> bucket_iterator;
 
   bucket_iterator bucket_begin(unsigned hash) {
     return bucket_iterator(Buckets + (hash & (NumBuckets-1)));
   }
-  
+
   bucket_iterator bucket_end(unsigned hash) {
     return bucket_iterator(Buckets + (hash & (NumBuckets-1)), true);
   }
-  
+
   /// GetOrInsertNode - If there is an existing simple Node exactly
   /// equal to the specified node, return it.  Otherwise, insert 'N' and
   /// return it instead.
   T *GetOrInsertNode(Node *N) {
     return static_cast<T *>(FoldingSetImpl::GetOrInsertNode(N));
   }
-  
+
   /// FindNodeOrInsertPos - Look up the node specified by ID.  If it exists,
   /// return it.  If not, return the insertion token that will make insertion
   /// faster.
@@ -302,7 +312,7 @@ protected:
   FoldingSetNode *NodePtr;
   FoldingSetIteratorImpl(void **Bucket);
   void advance();
-  
+
 public:
   bool operator==(const FoldingSetIteratorImpl &RHS) const {
     return NodePtr == RHS.NodePtr;
@@ -316,16 +326,16 @@ public:
 template<class T>
 class FoldingSetIterator : public FoldingSetIteratorImpl {
 public:
-  FoldingSetIterator(void **Bucket) : FoldingSetIteratorImpl(Bucket) {}
-  
+  explicit FoldingSetIterator(void **Bucket) : FoldingSetIteratorImpl(Bucket) {}
+
   T &operator*() const {
     return *static_cast<T*>(NodePtr);
   }
-  
+
   T *operator->() const {
     return static_cast<T*>(NodePtr);
   }
-  
+
   inline FoldingSetIterator& operator++() {          // Preincrement
     advance();
     return *this;
@@ -334,27 +344,27 @@ public:
     FoldingSetIterator tmp = *this; ++*this; return tmp;
   }
 };
-  
+
 //===----------------------------------------------------------------------===//
 /// FoldingSetBucketIteratorImpl - This is the common bucket iterator support
 ///  shared by all folding sets, which knows how to walk a particular bucket
 ///  of a folding set hash table.
-  
+
 class FoldingSetBucketIteratorImpl {
 protected:
   void *Ptr;
 
-  FoldingSetBucketIteratorImpl(void **Bucket);
-  
+  explicit FoldingSetBucketIteratorImpl(void **Bucket);
+
   FoldingSetBucketIteratorImpl(void **Bucket, bool)
-    : Ptr(reinterpret_cast<void*>(Bucket)) {}
+    : Ptr(Bucket) {}
 
   void advance() {
     void *Probe = static_cast<FoldingSetNode*>(Ptr)->getNextInBucket();
     uintptr_t x = reinterpret_cast<uintptr_t>(Probe) & ~0x1;
     Ptr = reinterpret_cast<void*>(x);
   }
-  
+
 public:
   bool operator==(const FoldingSetBucketIteratorImpl &RHS) const {
     return Ptr == RHS.Ptr;
@@ -363,29 +373,29 @@ public:
     return Ptr != RHS.Ptr;
   }
 };
-  
-  
+
+
 template<class T>
 class FoldingSetBucketIterator : public FoldingSetBucketIteratorImpl {
 public:
-  FoldingSetBucketIterator(void **Bucket) : 
+  explicit FoldingSetBucketIterator(void **Bucket) :
     FoldingSetBucketIteratorImpl(Bucket) {}
-  
-  FoldingSetBucketIterator(void **Bucket, bool) : 
+
+  FoldingSetBucketIterator(void **Bucket, bool) :
     FoldingSetBucketIteratorImpl(Bucket, true) {}
-  
-  T& operator*() const { return *static_cast<T*>(Ptr); }  
+
+  T& operator*() const { return *static_cast<T*>(Ptr); }
   T* operator->() const { return static_cast<T*>(Ptr); }
-  
+
   inline FoldingSetBucketIterator& operator++() { // Preincrement
     advance();
     return *this;
-  }          
+  }
   FoldingSetBucketIterator operator++(int) {      // Postincrement
     FoldingSetBucketIterator tmp = *this; ++*this; return tmp;
   }
 };
-  
+
 //===----------------------------------------------------------------------===//
 /// FoldingSetNodeWrapper - This template class is used to "wrap" arbitrary
 /// types in an enclosing object so that they can be inserted into FoldingSets.
@@ -393,32 +403,32 @@ template <typename T>
 class FoldingSetNodeWrapper : public FoldingSetNode {
   T data;
 public:
-  FoldingSetNodeWrapper(const T& x) : data(x) {}
+  explicit FoldingSetNodeWrapper(const T& x) : data(x) {}
   virtual ~FoldingSetNodeWrapper() {}
-  
+
   template<typename A1>
   explicit FoldingSetNodeWrapper(const A1& a1)
     : data(a1) {}
-  
+
   template <typename A1, typename A2>
   explicit FoldingSetNodeWrapper(const A1& a1, const A2& a2)
     : data(a1,a2) {}
-  
+
   template <typename A1, typename A2, typename A3>
   explicit FoldingSetNodeWrapper(const A1& a1, const A2& a2, const A3& a3)
     : data(a1,a2,a3) {}
-  
+
   template <typename A1, typename A2, typename A3, typename A4>
   explicit FoldingSetNodeWrapper(const A1& a1, const A2& a2, const A3& a3,
                                  const A4& a4)
     : data(a1,a2,a3,a4) {}
-  
+
   template <typename A1, typename A2, typename A3, typename A4, typename A5>
   explicit FoldingSetNodeWrapper(const A1& a1, const A2& a2, const A3& a3,
                                  const A4& a4, const A5& a5)
   : data(a1,a2,a3,a4,a5) {}
 
-  
+
   void Profile(FoldingSetNodeID& ID) { FoldingSetTrait<T>::Profile(data, ID); }
 
   T& getValue() { return data; }
@@ -428,8 +438,38 @@ public:
   operator const T&() const { return data; }
 };
 
+//===----------------------------------------------------------------------===//
+/// FastFoldingSetNode - This is a subclass of FoldingSetNode which stores
+/// a FoldingSetNodeID value rather than requiring the node to recompute it
+/// each time it is needed. This trades space for speed (which can be
+/// significant if the ID is long), and it also permits nodes to drop
+/// information that would otherwise only be required for recomputing an ID.
+class FastFoldingSetNode : public FoldingSetNode {
+  FoldingSetNodeID FastID;
+protected:
+  explicit FastFoldingSetNode(const FoldingSetNodeID &ID) : FastID(ID) {}
+public:
+  void Profile(FoldingSetNodeID& ID) { ID = FastID; }
+};
+
+//===----------------------------------------------------------------------===//
+// Partial specializations of FoldingSetTrait.
+
+template<typename T> struct FoldingSetTrait<T*> {
+  static inline void Profile(const T* X, FoldingSetNodeID& ID) {
+    ID.AddPointer(X);
+  }
+  static inline void Profile(T* X, FoldingSetNodeID& ID) {
+    ID.AddPointer(X);
+  }
+};
+
+template<typename T> struct FoldingSetTrait<const T*> {
+  static inline void Profile(const T* X, FoldingSetNodeID& ID) {
+    ID.AddPointer(X);
+  }
+};
+
 } // End of namespace llvm.
 
-
 #endif
-
