@@ -23,22 +23,40 @@
 
 namespace llvm {
 
-static inline std::string utohexstr(uint64_t X) {
-  char Buffer[40];
-  char *BufPtr = Buffer+39;
+/// hexdigit - Return the (uppercase) hexadecimal character for the
+/// given number \arg X (which should be less than 16).
+static inline char hexdigit(unsigned X) {
+  return X < 10 ? '0' + X : 'A' + X - 10;
+}
 
-  *BufPtr = 0;                  // Null terminate buffer...
-  if (X == 0) *--BufPtr = '0';  // Handle special case...
+/// utohex_buffer - Emit the specified number into the buffer specified by
+/// BufferEnd, returning a pointer to the start of the string.  This can be used
+/// like this: (note that the buffer must be large enough to handle any number):
+///    char Buffer[40];
+///    printf("0x%s", utohex_buffer(X, Buffer+40));
+///
+/// This should only be used with unsigned types.
+///
+template<typename IntTy>
+static inline char *utohex_buffer(IntTy X, char *BufferEnd) {
+  char *BufPtr = BufferEnd;
+  *--BufPtr = 0;      // Null terminate buffer.
+  if (X == 0) {
+    *--BufPtr = '0';  // Handle special case.
+    return BufPtr;
+  }
 
   while (X) {
     unsigned char Mod = static_cast<unsigned char>(X) & 15;
-    if (Mod < 10)
-      *--BufPtr = '0' + Mod;
-    else
-      *--BufPtr = 'A' + Mod-10;
+    *--BufPtr = hexdigit(Mod);
     X >>= 4;
   }
-  return std::string(BufPtr);
+  return BufPtr;
+}
+
+static inline std::string utohexstr(uint64_t X) {
+  char Buffer[40];
+  return utohex_buffer(X, Buffer+40);
 }
 
 static inline std::string utostr_32(uint32_t X, bool isNeg = false) {
@@ -61,18 +79,18 @@ static inline std::string utostr_32(uint32_t X, bool isNeg = false) {
 static inline std::string utostr(uint64_t X, bool isNeg = false) {
   if (X == uint32_t(X))
     return utostr_32(uint32_t(X), isNeg);
-  
+
   char Buffer[40];
   char *BufPtr = Buffer+39;
-  
+
   *BufPtr = 0;                  // Null terminate buffer...
   if (X == 0) *--BufPtr = '0';  // Handle special case...
-  
+
   while (X) {
     *--BufPtr = '0' + char(X % 10);
     X /= 10;
   }
-  
+
   if (isNeg) *--BufPtr = '-';   // Add negative sign...
   return std::string(BufPtr);
 }
@@ -85,10 +103,6 @@ static inline std::string itostr(int64_t X) {
     return utostr(static_cast<uint64_t>(X));
 }
 
-static inline std::string itohexstr(int64_t X) {
-  return utohexstr(static_cast<uint64_t>(X));
-}
-
 static inline std::string ftostr(double V) {
   char Buffer[200];
   sprintf(Buffer, "%20.6e", V);
@@ -123,7 +137,7 @@ static inline std::string UppercaseString(const std::string &S) {
 
 /// StringsEqualNoCase - Return true if the two strings are equal, ignoring
 /// case.
-static inline bool StringsEqualNoCase(const std::string &LHS, 
+static inline bool StringsEqualNoCase(const std::string &LHS,
                                       const std::string &RHS) {
   if (LHS.size() != RHS.size()) return false;
   for (unsigned i = 0, e = static_cast<unsigned>(LHS.size()); i != e; ++i)
@@ -133,7 +147,7 @@ static inline bool StringsEqualNoCase(const std::string &LHS,
 
 /// StringsEqualNoCase - Return true if the two strings are equal, ignoring
 /// case.
-static inline bool StringsEqualNoCase(const std::string &LHS, 
+static inline bool StringsEqualNoCase(const std::string &LHS,
                                       const char *RHS) {
   for (unsigned i = 0, e = static_cast<unsigned>(LHS.size()); i != e; ++i) {
     if (RHS[i] == 0) return false;  // RHS too short.
@@ -142,6 +156,25 @@ static inline bool StringsEqualNoCase(const std::string &LHS,
   return RHS[LHS.size()] == 0;  // Not too long?
 }
   
+/// StringsEqualNoCase - Return true if the two null-terminated C strings are
+///  equal, ignoring
+
+static inline bool StringsEqualNoCase(const char *LHS, const char *RHS,
+                                      unsigned len) {
+
+  for (unsigned i = 0; i < len; ++i) {
+    if (tolower(LHS[i]) != tolower(RHS[i]))
+      return false;
+    
+    // If RHS[i] == 0 then LHS[i] == 0 or otherwise we would have returned
+    // at the previous branch as tolower('\0') == '\0'.
+    if (RHS[i] == 0)
+      return true;
+  }
+  
+  return true;
+}
+
 /// CStrInCStrNoCase - Portable version of strcasestr.  Locates the first
 ///  occurance of c-string 's2' in string 's1', ignoring case.  Returns
 ///  NULL if 's2' cannot be found.
@@ -150,13 +183,13 @@ static inline const char* CStrInCStrNoCase(const char *s1, const char *s2) {
   // Are either strings NULL or empty?
   if (!s1 || !s2 || s1[0] == '\0' || s2[0] == '\0')
     return 0;
-  
+
   if (s1 == s2)
     return s1;
-  
+
   const char *I1=s1, *I2=s2;
-  
-  while (*I1 != '\0' || *I2 != '\0' )
+
+  while (*I1 != '\0' && *I2 != '\0' )
     if (tolower(*I1) != tolower(*I2)) { // No match.  Start over.
       ++s1; I1 = s1; I2 = s2;
     }
