@@ -27,20 +27,17 @@ class Function;
 class PMStack;
 
 class LoopPass : public Pass {
-
- public:
+public:
   explicit LoopPass(intptr_t pid) : Pass(pid) {}
+  explicit LoopPass(void *pid) : Pass(pid) {}
 
   // runOnLoop - This method should be implemented by the subclass to perform
-  // whatever action is necessary for the specfied Loop. 
-  virtual bool runOnLoop (Loop *L, LPPassManager &LPM) = 0;
-  virtual bool runOnFunctionBody (Function &F, LPPassManager &LPM) { 
-    return false; 
-  }
+  // whatever action is necessary for the specified Loop.
+  virtual bool runOnLoop(Loop *L, LPPassManager &LPM) = 0;
 
   // Initialization and finalization hooks.
-  virtual bool doInitialization(Loop *L, LPPassManager &LPM) { 
-    return false; 
+  virtual bool doInitialization(Loop *L, LPPassManager &LPM) {
+    return false;
   }
 
   // Finalization hook does not supply Loop because at this time
@@ -68,10 +65,10 @@ class LoopPass : public Pass {
   /// SimpleAnalysis - Provides simple interface to update analysis info
   /// maintained by various passes. Note, if required this interface can
   /// be extracted into a separate abstract class but it would require
-  /// additional use of multiple inheritance in Pass class hierarcy, someting
+  /// additional use of multiple inheritance in Pass class hierarchy, something
   /// we are trying to avoid.
 
-  /// Each loop pass can override these simple analysis hookss to update
+  /// Each loop pass can override these simple analysis hooks to update
   /// desired analysis information.
   /// cloneBasicBlockAnalysis - Clone analysis info associated with basic block.
   virtual void cloneBasicBlockAnalysis(BasicBlock *F, BasicBlock *T, Loop *L) {}
@@ -81,7 +78,6 @@ class LoopPass : public Pass {
 };
 
 class LPPassManager : public FunctionPass, public PMDataManager {
-
 public:
   static char ID;
   explicit LPPassManager(int Depth);
@@ -91,38 +87,31 @@ public:
   bool runOnFunction(Function &F);
 
   /// Pass Manager itself does not invalidate any analysis info.
-  // LPPassManager needs LoopInfo. 
-  void getAnalysisUsage(AnalysisUsage &Info) const; 
-  
+  // LPPassManager needs LoopInfo.
+  void getAnalysisUsage(AnalysisUsage &Info) const;
+
   virtual const char *getPassName() const {
     return "Loop Pass Manager";
   }
-  
-  // Print passes managed by this manager
-  void dumpPassStructure(unsigned Offset) {
-    llvm::cerr << std::string(Offset*2, ' ') << "Loop Pass Manager\n";
-    for (unsigned Index = 0; Index < getNumContainedPasses(); ++Index) {
-      Pass *P = getContainedPass(Index);
-      P->dumpPassStructure(Offset + 1);
-      dumpLastUses(P, Offset+1);
-    }
-  }
-  
+
+  /// Print passes managed by this manager
+  void dumpPassStructure(unsigned Offset);
+
   Pass *getContainedPass(unsigned N) {
-    assert ( N < PassVector.size() && "Pass number out of range!");
+    assert(N < PassVector.size() && "Pass number out of range!");
     Pass *FP = static_cast<Pass *>(PassVector[N]);
     return FP;
   }
 
-  virtual PassManagerType getPassManagerType() const { 
-    return PMT_LoopPassManager; 
+  virtual PassManagerType getPassManagerType() const {
+    return PMT_LoopPassManager;
   }
 
 public:
   // Delete loop from the loop queue and loop nest (LoopInfo).
   void deleteLoopFromQueue(Loop *L);
-  
-  // Inset loop into the loop nest(LoopInfo) and loop queue(LQ).
+
+  // Insert loop into the loop nest(LoopInfo) and loop queue(LQ).
   void insertLoop(Loop *L, Loop *ParentLoop);
 
   // Reoptimize this loop. LPPassManager will re-insert this loop into the
@@ -134,7 +123,7 @@ public:
   /// SimpleAnalysis - Provides simple interface to update analysis info
   /// maintained by various passes. Note, if required this interface can
   /// be extracted into a separate abstract class but it would require
-  /// additional use of multiple inheritance in Pass class hierarcy, someting
+  /// additional use of multiple inheritance in Pass class hierarchy, something
   /// we are trying to avoid.
 
   /// cloneBasicBlockSimpleAnalysis - Invoke cloneBasicBlockAnalysis hook for
@@ -144,6 +133,7 @@ public:
   /// deleteSimpleAnalysisValue - Invoke deleteAnalysisValue hook for all passes
   /// that implement simple analysis interface.
   void deleteSimpleAnalysisValue(Value *V, Loop *L);
+
 private:
   std::deque<Loop *> LQ;
   bool skipThisLoop;
