@@ -35,7 +35,7 @@ namespace CallingConv {
     // prototype exactly match.
 
     /// Fast - This calling convention attempts to make calls as fast as 
-    /// possible /// (e.g. by passing things in registers).
+    /// possible (e.g. by passing things in registers).
     Fast = 8,
 
     // Cold - This calling convention attempts to make code in the caller as
@@ -59,9 +59,16 @@ namespace CallingConv {
     /// stack cleaning.
     X86_FastCall = 65,
 
-    /// X86_SSEreg - The standard convention except that float and double
-    /// values are returned in XMM0 if SSE support is available.
-    X86_SSECall = 66
+    /// ARM_APCS - ARM Procedure Calling Standard calling convention (obsolete,
+    /// but still used on some targets).
+    ARM_APCS = 66,
+
+    /// ARM_AAPCS - ARM Architecture Procedure Calling Standard calling
+    /// convention (aka EABI). Soft float variant.
+    ARM_AAPCS = 67,
+
+    /// ARM_AAPCS_VFP - Same as ARM_AAPCS, but uses hard floating point ABI.
+    ARM_AAPCS_VFP = 68
   };
 } // End CallingConv namespace
 
