@@ -23,12 +23,13 @@
 #ifndef LLVM_SUPPORT_GRAPHWRITER_H
 #define LLVM_SUPPORT_GRAPHWRITER_H
 
-#include "llvm/Support/Debug.h"
 #include "llvm/Support/DOTGraphTraits.h"
+#include "llvm/Support/Streams.h"
 #include "llvm/ADT/GraphTraits.h"
 #include "llvm/System/Path.h"
 #include <fstream>
 #include <vector>
+#include <cassert>
 
 namespace llvm {
 
@@ -66,12 +67,23 @@ namespace DOT {  // Private functions...
   }
 }
 
-void DisplayGraph(const sys::Path& Filename);
-  
+namespace GraphProgram {
+   enum Name {
+      DOT,
+      FDP,
+      NEATO,
+      TWOPI,
+      CIRCO
+   };
+}
+   
+void DisplayGraph(const sys::Path& Filename, bool wait=true, GraphProgram::Name program = GraphProgram::DOT);
+
 template<typename GraphType>
 class GraphWriter {
   std::ostream &O;
   const GraphType &G;
+  bool ShortNames;
 
   typedef DOTGraphTraits<GraphType>           DOTTraits;
   typedef GraphTraits<GraphType>              GTraits;
@@ -79,19 +91,25 @@ class GraphWriter {
   typedef typename GTraits::nodes_iterator    node_iterator;
   typedef typename GTraits::ChildIteratorType child_iterator;
 public:
-  GraphWriter(std::ostream &o, const GraphType &g) : O(o), G(g) {}
+  GraphWriter(std::ostream &o, const GraphType &g, bool SN) :
+    O(o), G(g), ShortNames(SN) {}
 
   void writeHeader(const std::string &Name) {
-    if (Name.empty())
-      O << "digraph foo {\n";        // Graph name doesn't matter
+    std::string GraphName = DOTTraits::getGraphName(G);
+
+    if (!Name.empty())
+      O << "digraph \"" << DOT::EscapeString(Name) << "\" {\n";
+    else if (!GraphName.empty())
+      O << "digraph \"" << DOT::EscapeString(GraphName) << "\" {\n";
     else
-      O << "digraph " << Name << " {\n";
+      O << "digraph unnamed {\n";
 
     if (DOTTraits::renderGraphFromBottomUp())
       O << "\trankdir=\"BT\";\n";
 
-    std::string GraphName = DOTTraits::getGraphName(G);
-    if (!GraphName.empty())
+    if (!Name.empty())
+      O << "\tlabel=\"" << DOT::EscapeString(Name) << "\";\n";
+    else if (!GraphName.empty())
       O << "\tlabel=\"" << DOT::EscapeString(GraphName) << "\";\n";
     O << DOTTraits::getGraphProperties(G);
     O << "\n";
@@ -108,7 +126,7 @@ public:
          I != E; ++I)
       writeNode(*I);
   }
-  
+
   void writeNode(NodeType& Node) {
     writeNode(&Node);
   }
@@ -120,12 +138,12 @@ public:
   void writeNode(NodeType *Node) {
     std::string NodeAttributes = DOTTraits::getNodeAttributes(Node, G);
 
-    O << "\tNode" << reinterpret_cast<const void*>(Node) << " [shape=record,";
+    O << "\tNode" << static_cast<const void*>(Node) << " [shape=record,";
     if (!NodeAttributes.empty()) O << NodeAttributes << ",";
     O << "label=\"{";
 
     if (!DOTTraits::renderGraphFromBottomUp()) {
-      O << DOT::EscapeString(DOTTraits::getNodeLabel(Node, G));
+      O << DOT::EscapeString(DOTTraits::getNodeLabel(Node, G, ShortNames));
 
       // If we should include the address of the node in the label, do so now.
       if (DOTTraits::hasNodeAddressLabel(Node, G))
@@ -141,23 +159,37 @@ public:
 
       for (unsigned i = 0; EI != EE && i != 64; ++EI, ++i) {
         if (i) O << "|";
-        O << "<g" << i << ">" << DOTTraits::getEdgeSourceLabel(Node, EI);
+        O << "<s" << i << ">" << DOTTraits::getEdgeSourceLabel(Node, EI);
       }
 
       if (EI != EE)
-        O << "|<g64>truncated...";
+        O << "|<s64>truncated...";
       O << "}";
       if (DOTTraits::renderGraphFromBottomUp()) O << "|";
     }
 
     if (DOTTraits::renderGraphFromBottomUp()) {
-      O << DOT::EscapeString(DOTTraits::getNodeLabel(Node, G));
+      O << DOT::EscapeString(DOTTraits::getNodeLabel(Node, G, ShortNames));
 
       // If we should include the address of the node in the label, do so now.
       if (DOTTraits::hasNodeAddressLabel(Node, G))
         O << "|" << (void*)Node;
     }
 
+    if (DOTTraits::hasEdgeDestLabels()) {
+      O << "|{";
+
+      unsigned i = 0, e = DOTTraits::numEdgeDestLabels(Node);
+      for (; i != e && i != 64; ++i) {
+        if (i) O << "|";
+        O << "<d" << i << ">" << DOTTraits::getEdgeDestLabel(Node, i);
+      }
+
+      if (i != e)
+        O << "|<d64>truncated...";
+      O << "}";
+    }
+
     O << "}\"];\n";   // Finish printing the "node" line
 
     // Output all of the edges now
@@ -180,8 +212,8 @@ public:
         DestPort = static_cast<int>(Offset);
       }
 
-      emitEdge(reinterpret_cast<const void*>(Node), edgeidx,
-               reinterpret_cast<const void*>(TargetNode), DestPort,
+      emitEdge(static_cast<const void*>(Node), edgeidx,
+               static_cast<const void*>(TargetNode), DestPort,
                DOTTraits::getEdgeAttributes(Node, EI));
     }
   }
@@ -201,7 +233,7 @@ public:
 
       for (unsigned i = 0; i != NumEdgeSources; ++i) {
         if (i) O << "|";
-        O << "<g" << i << ">";
+        O << "<s" << i << ">";
         if (EdgeSourceLabels) O << (*EdgeSourceLabels)[i];
       }
       O << "}}";
@@ -218,10 +250,14 @@ public:
 
     O << "\tNode" << SrcNodeID;
     if (SrcNodePort >= 0)
-      O << ":g" << SrcNodePort;
-    O << " -> Node" << reinterpret_cast<const void*>(DestNodeID);
-    if (DestNodePort >= 0)
-      O << ":g" << DestNodePort;
+      O << ":s" << SrcNodePort;
+    O << " -> Node" << DestNodeID;
+    if (DestNodePort >= 0) {
+      if (DOTTraits::hasEdgeDestLabels())
+	O << ":d" << DestNodePort;
+      else
+	O << ":s" << DestNodePort;
+    }
 
     if (!Attrs.empty())
       O << "[" << Attrs << "]";
@@ -231,12 +267,14 @@ public:
 
 template<typename GraphType>
 std::ostream &WriteGraph(std::ostream &O, const GraphType &G,
-                         const std::string &Name = "") {
+                         bool ShortNames = false,
+                         const std::string &Name = "",
+                         const std::string &Title = "") {
   // Start the graph emission process...
-  GraphWriter<GraphType> W(O, G);
+  GraphWriter<GraphType> W(O, G, ShortNames);
 
   // Output the header for the graph...
-  W.writeHeader(Name);
+  W.writeHeader(Title);
 
   // Emit all of the nodes in the graph...
   W.writeNodes();
@@ -251,7 +289,8 @@ std::ostream &WriteGraph(std::ostream &O, const GraphType &G,
 
 template<typename GraphType>
 sys::Path WriteGraph(const GraphType &G,
-                     const std::string& Name, 
+                     const std::string& Name,
+                     bool ShortNames = false,
                      const std::string& Title = "") {
   std::string ErrMsg;
   sys::Path Filename = sys::Path::GetTemporaryDirectory(&ErrMsg);
@@ -266,50 +305,38 @@ sys::Path WriteGraph(const GraphType &G,
   }
 
   cerr << "Writing '" << Filename << "'... ";
-  
+
   std::ofstream O(Filename.c_str());
 
   if (O.good()) {
-    // Start the graph emission process...
-    GraphWriter<GraphType> W(O, G);
-
-    // Output the header for the graph...
-    W.writeHeader(Title);
-
-    // Emit all of the nodes in the graph...
-    W.writeNodes();
-
-    // Output any customizations on the graph
-    DOTGraphTraits<GraphType>::addCustomGraphFeatures(G, W);
-
-    // Output the end of the graph
-    W.writeFooter();
+    WriteGraph(O, G, ShortNames, Name, Title);
     cerr << " done. \n";
 
     O.close();
-    
   } else {
     cerr << "error opening file for writing!\n";
     Filename.clear();
   }
-  
+
   return Filename;
 }
-  
+
 /// ViewGraph - Emit a dot graph, run 'dot', run gv on the postscript file,
 /// then cleanup.  For use from the debugger.
 ///
 template<typename GraphType>
-void ViewGraph(const GraphType& G, 
-               const std::string& Name, 
-               const std::string& Title = "") {
-  sys::Path Filename =  WriteGraph(G, Name, Title);
+void ViewGraph(const GraphType& G,
+               const std::string& Name,
+               bool ShortNames = false,
+               const std::string& Title = "",
+               GraphProgram::Name Program = GraphProgram::DOT) {
+  sys::Path Filename =  WriteGraph(G, Name, ShortNames, Title);
 
   if (Filename.isEmpty()) {
     return;
   }
-  
-  DisplayGraph(Filename);
+
+  DisplayGraph(Filename, true, Program);
 }
 
 } // End llvm namespace
