@@ -18,7 +18,7 @@
 
 namespace llvm {
 
-// NOTE: The following support functions use the _32/_64 extensions instead of  
+// NOTE: The following support functions use the _32/_64 extensions instead of
 // type overloading so that signed and unsigned integers can be used without
 // ambiguity.
 
@@ -33,23 +33,33 @@ inline uint32_t Lo_32(uint64_t Value) {
 }
 
 /// is?Type - these functions produce optimal testing for integer data types.
-inline bool isInt8  (int64_t Value) { 
-  return static_cast<int8_t>(Value) == Value; 
+inline bool isInt8  (int64_t Value) {
+  return static_cast<int8_t>(Value) == Value;
 }
-inline bool isUInt8 (int64_t Value) { 
-  return static_cast<uint8_t>(Value) == Value; 
+inline bool isUInt8 (int64_t Value) {
+  return static_cast<uint8_t>(Value) == Value;
 }
-inline bool isInt16 (int64_t Value) { 
-  return static_cast<int16_t>(Value) == Value; 
+inline bool isInt16 (int64_t Value) {
+  return static_cast<int16_t>(Value) == Value;
 }
-inline bool isUInt16(int64_t Value) { 
-  return static_cast<uint16_t>(Value) == Value; 
+inline bool isUInt16(int64_t Value) {
+  return static_cast<uint16_t>(Value) == Value;
 }
-inline bool isInt32 (int64_t Value) { 
-  return static_cast<int32_t>(Value) == Value; 
+inline bool isInt32 (int64_t Value) {
+  return static_cast<int32_t>(Value) == Value;
 }
-inline bool isUInt32(int64_t Value) { 
-  return static_cast<uint32_t>(Value) == Value; 
+inline bool isUInt32(int64_t Value) {
+  return static_cast<uint32_t>(Value) == Value;
+}
+
+template<unsigned N>
+inline bool isInt(int64_t x) {
+  return N >= 64 || (-(INT64_C(1)<<(N-1)) <= x && x < (INT64_C(1)<<(N-1)));
+}
+
+template<unsigned N>
+inline bool isUint(uint64_t x) {
+  return N >= 64 || x < (UINT64_C(1)<<N);
 }
 
 /// isMask_32 - This function returns true if the argument is a sequence of ones
@@ -66,20 +76,20 @@ inline bool isMask_64(uint64_t Value) {
   return Value && ((Value + 1) & Value) == 0;
 }
 
-/// isShiftedMask_32 - This function returns true if the argument contains a  
+/// isShiftedMask_32 - This function returns true if the argument contains a
 /// sequence of ones with the remainder zero (32 bit version.)
 /// Ex. isShiftedMask_32(0x0000FF00U) == true.
 inline bool isShiftedMask_32(uint32_t Value) {
   return isMask_32((Value - 1) | Value);
 }
 
-/// isShiftedMask_64 - This function returns true if the argument contains a  
+/// isShiftedMask_64 - This function returns true if the argument contains a
 /// sequence of ones with the remainder zero (64 bit version.)
 inline bool isShiftedMask_64(uint64_t Value) {
   return isMask_64((Value - 1) | Value);
 }
 
-/// isPowerOf2_32 - This function returns true if the argument is a power of 
+/// isPowerOf2_32 - This function returns true if the argument is a power of
 /// two > 0. Ex. isPowerOf2_32(0x00100000U) == true (32 bit edition.)
 inline bool isPowerOf2_32(uint32_t Value) {
   return Value && !(Value & (Value - 1));
@@ -108,7 +118,7 @@ inline uint16_t ByteSwap_16(uint16_t Value) {
 /// ByteSwap_32 - This function returns a byte-swapped representation of the
 /// 32-bit argument, Value.
 inline uint32_t ByteSwap_32(uint32_t Value) {
-#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
+#if (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)) && !defined(__ICC)
   return __builtin_bswap32(Value);
 #elif defined(_MSC_VER) && !defined(_DEBUG)
   return _byteswap_ulong(Value);
@@ -124,7 +134,7 @@ inline uint32_t ByteSwap_32(uint32_t Value) {
 /// ByteSwap_64 - This function returns a byte-swapped representation of the
 /// 64-bit argument, Value.
 inline uint64_t ByteSwap_64(uint64_t Value) {
-#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
+#if (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)) && !defined(__ICC)
   return __builtin_bswap64(Value);
 #elif defined(_MSC_VER) && !defined(_DEBUG)
   return _byteswap_uint64(Value);
@@ -172,7 +182,7 @@ inline unsigned CountLeadingOnes_32(uint32_t Value) {
 }
 
 /// CountLeadingZeros_64 - This function performs the platform optimal form
-/// of counting the number of zeros from the most significant bit to the first 
+/// of counting the number of zeros from the most significant bit to the first
 /// one bit (64 bit edition.)
 /// Returns 64 if the word is zero.
 inline unsigned CountLeadingZeros_64(uint64_t Value) {
@@ -216,7 +226,7 @@ inline unsigned CountLeadingZeros_64(uint64_t Value) {
 }
 
 /// CountLeadingOnes_64 - This function performs the operation
-/// of counting the number of ones from the most significant bit to the first 
+/// of counting the number of ones from the most significant bit to the first
 /// zero bit (64 bit edition.)
 /// Returns 64 if the word is all ones.
 inline unsigned CountLeadingOnes_64(uint64_t Value) {
@@ -249,7 +259,7 @@ inline unsigned CountTrailingOnes_32(uint32_t Value) {
 }
 
 /// CountTrailingZeros_64 - This function performs the platform optimal form
-/// of counting the number of zeros from the least significant bit to the first 
+/// of counting the number of zeros from the least significant bit to the first
 /// one bit (64 bit edition.)
 /// Returns 64 if the word is zero.
 inline unsigned CountTrailingZeros_64(uint64_t Value) {
@@ -268,7 +278,7 @@ inline unsigned CountTrailingZeros_64(uint64_t Value) {
 }
 
 /// CountTrailingOnes_64 - This function performs the operation
-/// of counting the number of ones from the least significant bit to the first 
+/// of counting the number of ones from the least significant bit to the first
 /// zero bit (64 bit edition.)
 /// Returns 64 if the word is all ones.
 inline unsigned CountTrailingOnes_64(uint64_t Value) {
@@ -301,14 +311,14 @@ inline unsigned CountPopulation_64(uint64_t Value) {
 #endif
 }
 
-/// Log2_32 - This function returns the floor log base 2 of the specified value, 
+/// Log2_32 - This function returns the floor log base 2 of the specified value,
 /// -1 if the value is zero. (32 bit edition.)
 /// Ex. Log2_32(32) == 5, Log2_32(1) == 0, Log2_32(0) == -1, Log2_32(6) == 2
 inline unsigned Log2_32(uint32_t Value) {
   return 31 - CountLeadingZeros_32(Value);
 }
 
-/// Log2_64 - This function returns the floor log base 2 of the specified value, 
+/// Log2_64 - This function returns the floor log base 2 of the specified value,
 /// -1 if the value is zero. (64 bit edition.)
 inline unsigned Log2_64(uint64_t Value) {
   return 63 - CountLeadingZeros_64(Value);
@@ -321,8 +331,8 @@ inline unsigned Log2_32_Ceil(uint32_t Value) {
   return 32-CountLeadingZeros_32(Value-1);
 }
 
-/// Log2_64 - This function returns the ceil log base 2 of the specified value, 
-/// 64 if the value is zero. (64 bit edition.)
+/// Log2_64_Ceil - This function returns the ceil log base 2 of the specified
+/// value, 64 if the value is zero. (64 bit edition.)
 inline unsigned Log2_64_Ceil(uint64_t Value) {
   return 64-CountLeadingZeros_64(Value-1);
 }
@@ -337,7 +347,7 @@ inline uint64_t GreatestCommonDivisor64(uint64_t A, uint64_t B) {
   }
   return A;
 }
-  
+
 /// BitsToDouble - This function takes a 64-bit integer and returns the bit
 /// equivalent double.
 inline double BitsToDouble(uint64_t Bits) {
@@ -361,7 +371,9 @@ inline float BitsToFloat(uint32_t Bits) {
 }
 
 /// DoubleToBits - This function takes a double and returns the bit
-/// equivalent 64-bit integer.
+/// equivalent 64-bit integer.  Note that copying doubles around
+/// changes the bits of NaNs on some hosts, notably x86, so this
+/// routine cannot be used if these bits are needed.
 inline uint64_t DoubleToBits(double Double) {
   union {
     uint64_t L;
@@ -372,7 +384,9 @@ inline uint64_t DoubleToBits(double Double) {
 }
 
 /// FloatToBits - This function takes a float and returns the bit
-/// equivalent 32-bit integer.
+/// equivalent 32-bit integer.  Note that copying floats around
+/// changes the bits of NaNs on some hosts, notably x86, so this
+/// routine cannot be used if these bits are needed.
 inline uint32_t FloatToBits(float Float) {
   union {
     uint32_t I;
@@ -396,7 +410,38 @@ static inline uint64_t MinAlign(uint64_t A, uint64_t B) {
   // The largest power of 2 that divides both A and B.
   return (A | B) & -(A | B);
 }
-  
+
+/// NextPowerOf2 - Returns the next power of two (in 64-bits)
+/// that is strictly greater than A.  Returns zero on overflow.
+static inline uint64_t NextPowerOf2(uint64_t A) {
+  A |= (A >> 1);
+  A |= (A >> 2);
+  A |= (A >> 4);
+  A |= (A >> 8);
+  A |= (A >> 16);
+  A |= (A >> 32);
+  return A + 1;
+}
+
+/// RoundUpToAlignment - Returns the next integer (mod 2**64) that is
+/// greater than or equal to \arg Value and is a multiple of \arg
+/// Align. Align must be non-zero.
+///
+/// Examples:
+/// RoundUpToAlignment(5, 8) = 8
+/// RoundUpToAlignment(17, 8) = 24
+/// RoundUpToAlignment(~0LL, 8) = 0
+inline uint64_t RoundUpToAlignment(uint64_t Value, uint64_t Align) {
+  return ((Value + Align - 1) / Align) * Align;
+}
+
+/// abs64 - absolute value of a 64-bit int.  Not all environments support
+/// "abs" on whatever their name for the 64-bit int type is.  The absolute
+/// value of the largest negative number is undefined, as with "abs".
+inline int64_t abs64(int64_t x) {
+  return (x < 0) ? -x : x;
+}
+
 } // End llvm namespace
 
 #endif
