@@ -14,10 +14,12 @@
 #ifndef LLVM_SYSTEM_HOST_H
 #define LLVM_SYSTEM_HOST_H
 
+#include <string>
+
 namespace llvm {
 namespace sys {
 
-  inline bool littleEndianHost() {
+  inline bool isLittleEndianHost() {
     union {
       int i;
       char c;
@@ -26,10 +28,19 @@ namespace sys {
     return c;
   }
 
-  inline bool bigEndianHost() {
-    return !littleEndianHost();
+  inline bool isBigEndianHost() {
+    return !isLittleEndianHost();
   }
 
+  /// getHostTriple() - Return the target triple of the running
+  /// system.
+  ///
+  /// The target triple is a string in the format of:
+  ///   CPU_TYPE-VENDOR-OPERATING_SYSTEM
+  /// or
+  ///   CPU_TYPE-VENDOR-KERNEL-OPERATING_SYSTEM
+  std::string getHostTriple();
+
 }
 }
 
