@@ -16,56 +16,27 @@
 #ifndef LLVM_TARGET_ASM_INFO_H
 #define LLVM_TARGET_ASM_INFO_H
 
-#include "llvm/CodeGen/MachineFunctionPass.h"
 #include "llvm/Support/DataTypes.h"
+#include <cassert>
+#include <string>
 
 namespace llvm {
-  // DWARF encoding query type
-  namespace DwarfEncoding {
-    enum Target {
-      Data       = 0,
-      CodeLabels = 1,
-      Functions  = 2
-    };
-  }
-
-  class TargetMachine;
-  class CallInst;
-
+  template <typename T> class SmallVectorImpl;
+  
   /// TargetAsmInfo - This class is intended to be used as a base class for asm
   /// properties and features specific to the target.
+  namespace ExceptionHandling { enum ExceptionsType { None, Dwarf, SjLj }; }
+
   class TargetAsmInfo {
   protected:
     //===------------------------------------------------------------------===//
     // Properties to be set by the target writer, used to configure asm printer.
     //
-    
-    /// TextSection - Section directive for standard text.
-    ///
-    const char *TextSection;              // Defaults to ".text".
-    
-    /// DataSection - Section directive for standard data.
-    ///
-    const char *DataSection;              // Defaults to ".data".
-
-    /// BSSSection - Section directive for uninitialized data.  Null if this
-    /// target doesn't support a BSS section.
-    ///
-    const char *BSSSection;               // Default to ".bss".
-
-    /// TLSDataSection - Section directive for Thread Local data.
-    ///
-    const char *TLSDataSection;// Defaults to ".section .tdata,"awT",@progbits".
-
-    /// TLSBSSSection - Section directive for Thread Local uninitialized data.
-    /// Null if this target doesn't support a BSS section.
-    ///
-    const char *TLSBSSSection;// Default to ".section .tbss,"awT",@nobits".
 
     /// ZeroFillDirective - Directive for emitting a global to the ZeroFill
     /// section on this target.  Null if this target doesn't support zerofill.
     const char *ZeroFillDirective;        // Default is null.
-    
+
     /// NonexecutableStackDirective - Directive for declaring to the
     /// linker and beyond that the emitted code does not require stack
     /// memory to be executable.
@@ -95,6 +66,10 @@ namespace llvm {
     /// measure inline asm instructions.
     char SeparatorChar;                   // Defaults to ';'
 
+    /// CommentColumn - This indicates the comment num (zero-based) at
+    /// which asm comments should be printed.
+    unsigned CommentColumn;               // Defaults to 60
+
     /// CommentString - This indicates the comment character used by the
     /// assembler.
     const char *CommentString;            // Defaults to "#"
@@ -104,10 +79,15 @@ namespace llvm {
     const char *GlobalPrefix;             // Defaults to ""
 
     /// PrivateGlobalPrefix - This prefix is used for globals like constant
-    /// pool entries that are completely private to the .o file and should not
+    /// pool entries that are completely private to the .s file and should not
     /// have names in the .o file.  This is often "." or "L".
     const char *PrivateGlobalPrefix;      // Defaults to "."
     
+    /// LinkerPrivateGlobalPrefix - This prefix is used for symbols that should
+    /// be passed through the assembler but be removed by the linker.  This
+    /// is "l" on Darwin, currently used for some ObjC metadata.
+    const char *LinkerPrivateGlobalPrefix;      // Defaults to ""
+    
     /// JumpTableSpecialLabelPrefix - If not null, a extra (dead) label is
     /// emitted before jump tables with the specified prefix.
     const char *JumpTableSpecialLabelPrefix;  // Default to null.
@@ -120,10 +100,6 @@ namespace llvm {
 
     /// FunctionAddrPrefix/Suffix - If these are nonempty, these strings
     /// will enclose any GlobalVariable that points to a function.
-    /// For example, this is used by the IA64 backend to materialize
-    /// function descriptors, by decorating the ".data8" object with the
-    /// @verbatim @fptr( ) @endverbatim
-    /// link-relocation operator.
     ///
     const char *FunctionAddrPrefix;       // Defaults to ""
     const char *FunctionAddrSuffix;       // Defaults to ""
@@ -147,6 +123,10 @@ namespace llvm {
     /// AssemblerDialect - Which dialect of an assembler variant to use.
     unsigned AssemblerDialect;            // Defaults to 0
 
+    /// AllowQuotesInName - This is true if the assembler allows for complex
+    /// symbol names to be surrounded in quotes.  This defaults to false.
+    bool AllowQuotesInName;
+    
     //===--- Data Emission Directives -------------------------------------===//
 
     /// ZeroDirective - this should be set to the directive used to get some
@@ -173,6 +153,23 @@ namespace llvm {
     const char *Data32bitsDirective;      // Defaults to "\t.long\t"
     const char *Data64bitsDirective;      // Defaults to "\t.quad\t"
 
+    /// getDataASDirective - Return the directive that should be used to emit
+    /// data of the specified size to the specified numeric address space.
+    virtual const char *getDataASDirective(unsigned Size, unsigned AS) const {
+      assert(AS != 0 && "Don't know the directives for default addr space");
+      return NULL;
+    }
+
+    /// SunStyleELFSectionSwitchSyntax - This is true if this target uses "Sun
+    /// Style" syntax for section switching ("#alloc,#write" etc) instead of the
+    /// normal ELF syntax (,"a,w") in .section directives.
+    bool SunStyleELFSectionSwitchSyntax;   // Defaults to false.
+
+    /// UsesELFSectionDirectiveForBSS - This is true if this target uses ELF
+    /// '.section' directive before the '.bss' one. It's used for PPC/Linux 
+    /// which doesn't support the '.bss' directive only.
+    bool UsesELFSectionDirectiveForBSS;  // Defaults to false.
+    
     //===--- Alignment Information ----------------------------------------===//
 
     /// AlignDirective - The directive used to emit round up to an alignment
@@ -192,69 +189,22 @@ namespace llvm {
 
     //===--- Section Switching Directives ---------------------------------===//
     
-    /// SwitchToSectionDirective - This is the directive used when we want to
-    /// emit a global to an arbitrary section.  The section name is emited after
-    /// this.
-    const char *SwitchToSectionDirective; // Defaults to "\t.section\t"
-    
-    /// TextSectionStartSuffix - This is printed after each start of section
-    /// directive for text sections.
-    const char *TextSectionStartSuffix;   // Defaults to "".
-
-    /// DataSectionStartSuffix - This is printed after each start of section
-    /// directive for data sections.
-    const char *DataSectionStartSuffix;   // Defaults to "".
-    
-    /// SectionEndDirectiveSuffix - If non-null, the asm printer will close each
-    /// section with the section name and this suffix printed.
-    const char *SectionEndDirectiveSuffix;// Defaults to null.
-    
-    /// ConstantPoolSection - This is the section that we SwitchToSection right
-    /// before emitting the constant pool for a function.
-    const char *ConstantPoolSection;      // Defaults to "\t.section .rodata"
-
-    /// JumpTableDataSection - This is the section that we SwitchToSection right
-    /// before emitting the jump tables for a function when the relocation model
-    /// is not PIC.
-    const char *JumpTableDataSection;     // Defaults to "\t.section .rodata"
-    
-    /// JumpTableDirective - if non-null, the directive to emit before a jump
-    /// table.
+    /// JumpTableDirective - if non-null, the directive to emit before jump
+    /// table entries.  FIXME: REMOVE THIS.
     const char *JumpTableDirective;
+    const char *PICJumpTableDirective;
 
-    /// CStringSection - If not null, this allows for special handling of
-    /// cstring constants (null terminated string that does not contain any
-    /// other null bytes) on this target. This is commonly supported as
-    /// ".cstring".
-    const char *CStringSection;           // Defaults to NULL
-
-    /// StaticCtorsSection - This is the directive that is emitted to switch to
-    /// a section to emit the static constructor list.
-    /// Defaults to "\t.section .ctors,\"aw\",@progbits".
-    const char *StaticCtorsSection;
-
-    /// StaticDtorsSection - This is the directive that is emitted to switch to
-    /// a section to emit the static destructor list.
-    /// Defaults to "\t.section .dtors,\"aw\",@progbits".
-    const char *StaticDtorsSection;
-
-    /// FourByteConstantSection, EightByteConstantSection,
-    /// SixteenByteConstantSection - These are special sections where we place
-    /// 4-, 8-, and 16- byte constant literals.
-    const char *FourByteConstantSection;
-    const char *EightByteConstantSection;
-    const char *SixteenByteConstantSection;
-
-    /// ReadOnlySection - This is the directive that is emitted to switch to a
-    /// read-only section for constant data (e.g. data declared const,
-    /// jump tables).
-    const char *ReadOnlySection;          // Defaults to NULL
 
     //===--- Global Variable Emission Directives --------------------------===//
     
     /// GlobalDirective - This is the directive used to declare a global entity.
     ///
     const char *GlobalDirective;          // Defaults to NULL.
+
+    /// ExternDirective - This is the directive used to declare external 
+    /// globals.
+    ///
+    const char *ExternDirective;          // Defaults to NULL.
     
     /// SetDirective - This is the name of a directive that can be used to tell
     /// the assembler to set the value of a variable to some expression.
@@ -276,7 +226,11 @@ namespace llvm {
     /// HasDotTypeDotSizeDirective - True if the target has .type and .size
     /// directives, this is true for most ELF targets.
     bool HasDotTypeDotSizeDirective;      // Defaults to true.
-    
+
+    /// HasSingleParameterDotFile - True if the target has a single parameter
+    /// .file directive, this is true for ELF targets.
+    bool HasSingleParameterDotFile;      // Defaults to true.
+
     /// UsedDirective - This directive, if non-null, is used to declare a global
     /// as being used somehow that the assembler can't see.  This prevents dead
     /// code elimination on some targets.
@@ -311,25 +265,34 @@ namespace llvm {
     /// HasLEB128 - True if target asm supports leb128 directives.
     ///
     bool HasLEB128; // Defaults to false.
-    
+
     /// hasDotLocAndDotFile - True if target asm supports .loc and .file
     /// directives for emitting debugging information.
     ///
     bool HasDotLocAndDotFile; // Defaults to false.
-    
+
     /// SupportsDebugInformation - True if target supports emission of debugging
     /// information.
     bool SupportsDebugInformation;
-        
+
     /// SupportsExceptionHandling - True if target supports
     /// exception handling.
     ///
-    bool SupportsExceptionHandling; // Defaults to false.
-    
+    // Defaults to None
+    ExceptionHandling::ExceptionsType ExceptionsType;
+
     /// RequiresFrameSection - true if the Dwarf2 output needs a frame section
     ///
     bool DwarfRequiresFrameSection; // Defaults to true.
 
+    /// DwarfUsesInlineInfoSection - True if DwarfDebugInlineSection is used to
+    /// encode inline subroutine information.
+    bool DwarfUsesInlineInfoSection; // Defaults to false.
+
+    /// Is_EHSymbolPrivate - If set, the "_foo.eh" is made private so that it
+    /// doesn't show up in the symbol table of the object file.
+    bool Is_EHSymbolPrivate;                // Defaults to true.
+
     /// GlobalEHDirective - This is the directive used to make exception frame
     /// tables globally visible.
     ///
@@ -342,103 +305,48 @@ namespace llvm {
     /// DwarfSectionOffsetDirective - Special section offset directive.
     const char* DwarfSectionOffsetDirective; // Defaults to NULL
     
-    /// DwarfAbbrevSection - Section directive for Dwarf abbrev.
-    ///
-    const char *DwarfAbbrevSection; // Defaults to ".debug_abbrev".
-
-    /// DwarfInfoSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfInfoSection; // Defaults to ".debug_info".
-
-    /// DwarfLineSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfLineSection; // Defaults to ".debug_line".
-    
-    /// DwarfFrameSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfFrameSection; // Defaults to ".debug_frame".
-    
-    /// DwarfPubNamesSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfPubNamesSection; // Defaults to ".debug_pubnames".
-    
-    /// DwarfPubTypesSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfPubTypesSection; // Defaults to ".debug_pubtypes".
-    
-    /// DwarfStrSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfStrSection; // Defaults to ".debug_str".
-
-    /// DwarfLocSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfLocSection; // Defaults to ".debug_loc".
-
-    /// DwarfARangesSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfARangesSection; // Defaults to ".debug_aranges".
-
-    /// DwarfRangesSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfRangesSection; // Defaults to ".debug_ranges".
-
-    /// DwarfMacInfoSection - Section directive for Dwarf info.
-    ///
-    const char *DwarfMacInfoSection; // Defaults to ".debug_macinfo".
-    
-    /// DwarfEHFrameSection - Section directive for Exception frames.
-    ///
-    const char *DwarfEHFrameSection; // Defaults to ".eh_frame".
-    
-    /// DwarfExceptionSection - Section directive for Exception table.
-    ///
-    const char *DwarfExceptionSection; // Defaults to ".gcc_except_table".
-     
-
     //===--- CBE Asm Translation Table -----------------------------------===//
 
     const char *const *AsmTransCBE; // Defaults to empty
 
   public:
-    TargetAsmInfo();
+    explicit TargetAsmInfo();
     virtual ~TargetAsmInfo();
 
-    /// Measure the specified inline asm to determine an approximation of its
-    /// length.
-    virtual unsigned getInlineAsmLength(const char *Str) const;
+    /// getSLEB128Size - Compute the number of bytes required for a signed
+    /// leb128 value.
+    static unsigned getSLEB128Size(int Value);
 
-    /// ExpandInlineAsm - This hook allows the target to expand an inline asm
-    /// call to be explicit llvm code if it wants to.  This is useful for
-    /// turning simple inline asms into LLVM intrinsics, which gives the
-    /// compiler more information about the behavior of the code.
-    virtual bool ExpandInlineAsm(CallInst *CI) const {
-      return false;
+    /// getULEB128Size - Compute the number of bytes required for an unsigned
+    /// leb128 value.
+    static unsigned getULEB128Size(unsigned Value);
+
+    // Data directive accessors.
+    //
+    const char *getData8bitsDirective(unsigned AS = 0) const {
+      return AS == 0 ? Data8bitsDirective : getDataASDirective(8, AS);
+    }
+    const char *getData16bitsDirective(unsigned AS = 0) const {
+      return AS == 0 ? Data16bitsDirective : getDataASDirective(16, AS);
+    }
+    const char *getData32bitsDirective(unsigned AS = 0) const {
+      return AS == 0 ? Data32bitsDirective : getDataASDirective(32, AS);
+    }
+    const char *getData64bitsDirective(unsigned AS = 0) const {
+      return AS == 0 ? Data64bitsDirective : getDataASDirective(64, AS);
     }
 
-    /// PreferredEHDataFormat - This hook allows the target to select data
-    /// format used for encoding pointers in exception handling data. Reason is
-    /// 0 for data, 1 for code labels, 2 for function pointers. Global is true
-    /// if the symbol can be relocated.
-    virtual unsigned PreferredEHDataFormat(DwarfEncoding::Target Reason,
-                                           bool Global) const;
     
+    bool usesSunStyleELFSectionSwitchSyntax() const {
+      return SunStyleELFSectionSwitchSyntax;
+    }
+    
+    bool usesELFSectionDirectiveForBSS() const {
+      return UsesELFSectionDirectiveForBSS;
+    }
+
     // Accessors.
     //
-    const char *getTextSection() const {
-      return TextSection;
-    }
-    const char *getDataSection() const {
-      return DataSection;
-    }
-    const char *getBSSSection() const {
-      return BSSSection;
-    }
-    const char *getTLSDataSection() const {
-      return TLSDataSection;
-    }
-    const char *getTLSBSSSection() const {
-      return TLSBSSSection;
-    }
     const char *getZeroFillDirective() const {
       return ZeroFillDirective;
     }
@@ -448,12 +356,18 @@ namespace llvm {
     bool needsSet() const {
       return NeedsSet;
     }
+    unsigned getMaxInstLength() const {
+      return MaxInstLength;
+    }
     const char *getPCSymbol() const {
       return PCSymbol;
     }
     char getSeparatorChar() const {
       return SeparatorChar;
     }
+    unsigned getCommentColumn() const {
+      return CommentColumn;
+    }
     const char *getCommentString() const {
       return CommentString;
     }
@@ -463,6 +377,9 @@ namespace llvm {
     const char *getPrivateGlobalPrefix() const {
       return PrivateGlobalPrefix;
     }
+    const char *getLinkerPrivateGlobalPrefix() const {
+      return LinkerPrivateGlobalPrefix;
+    }
     const char *getJumpTableSpecialLabelPrefix() const {
       return JumpTableSpecialLabelPrefix;
     }
@@ -496,6 +413,9 @@ namespace llvm {
     unsigned getAssemblerDialect() const {
       return AssemblerDialect;
     }
+    bool doesAllowQuotesInName() const {
+      return AllowQuotesInName;
+    }
     const char *getZeroDirective() const {
       return ZeroDirective;
     }
@@ -508,20 +428,8 @@ namespace llvm {
     const char *getAscizDirective() const {
       return AscizDirective;
     }
-    const char *getData8bitsDirective() const {
-      return Data8bitsDirective;
-    }
-    const char *getData16bitsDirective() const {
-      return Data16bitsDirective;
-    }
-    const char *getData32bitsDirective() const {
-      return Data32bitsDirective;
-    }
-    const char *getData64bitsDirective() const {
-      return Data64bitsDirective;
-    }
-    const char *getJumpTableDirective() const {
-      return JumpTableDirective;
+    const char *getJumpTableDirective(bool isPIC) const {
+      return isPIC ? PICJumpTableDirective : JumpTableDirective;
     }
     const char *getAlignDirective() const {
       return AlignDirective;
@@ -532,48 +440,12 @@ namespace llvm {
     unsigned getTextAlignFillValue() const {
       return TextAlignFillValue;
     }
-    const char *getSwitchToSectionDirective() const {
-      return SwitchToSectionDirective;
-    }
-    const char *getTextSectionStartSuffix() const {
-      return TextSectionStartSuffix;
-    }
-    const char *getDataSectionStartSuffix() const {
-      return DataSectionStartSuffix;
-    }
-    const char *getSectionEndDirectiveSuffix() const {
-      return SectionEndDirectiveSuffix;
-    }
-    const char *getConstantPoolSection() const {
-      return ConstantPoolSection;
-    }
-    const char *getJumpTableDataSection() const {
-      return JumpTableDataSection;
-    }
-    const char *getCStringSection() const {
-      return CStringSection;
-    }
-    const char *getStaticCtorsSection() const {
-      return StaticCtorsSection;
-    }
-    const char *getStaticDtorsSection() const {
-      return StaticDtorsSection;
-    }
-    const char *getFourByteConstantSection() const {
-      return FourByteConstantSection;
-    }
-    const char *getEightByteConstantSection() const {
-      return EightByteConstantSection;
-    }
-    const char *getSixteenByteConstantSection() const {
-      return SixteenByteConstantSection;
-    }
-    const char *getReadOnlySection() const {
-      return ReadOnlySection;
-    }
     const char *getGlobalDirective() const {
       return GlobalDirective;
     }
+    const char *getExternDirective() const {
+      return ExternDirective;
+    }
     const char *getSetDirective() const {
       return SetDirective;
     }
@@ -589,6 +461,9 @@ namespace llvm {
     bool hasDotTypeDotSizeDirective() const {
       return HasDotTypeDotSizeDirective;
     }
+    bool hasSingleParameterDotFile() const {
+      return HasSingleParameterDotFile;
+    }
     const char *getUsedDirective() const {
       return UsedDirective;
     }
@@ -620,11 +495,20 @@ namespace llvm {
       return SupportsDebugInformation;
     }
     bool doesSupportExceptionHandling() const {
-      return SupportsExceptionHandling;
+      return ExceptionsType != ExceptionHandling::None;
+    }
+    ExceptionHandling::ExceptionsType getExceptionHandlingType() const {
+      return ExceptionsType;
     }
     bool doesDwarfRequireFrameSection() const {
       return DwarfRequiresFrameSection;
     }
+    bool doesDwarfUsesInlineInfoSection() const {
+      return DwarfUsesInlineInfoSection;
+    }
+    bool is_EHSymbolPrivate() const {
+      return Is_EHSymbolPrivate;
+    }
     const char *getGlobalEHDirective() const {
       return GlobalEHDirective;
     }
@@ -633,45 +517,6 @@ namespace llvm {
     }
     const char *getDwarfSectionOffsetDirective() const {
       return DwarfSectionOffsetDirective;
-    }    
-    const char *getDwarfAbbrevSection() const {
-      return DwarfAbbrevSection;
-    }
-    const char *getDwarfInfoSection() const {
-      return DwarfInfoSection;
-    }
-    const char *getDwarfLineSection() const {
-      return DwarfLineSection;
-    }
-    const char *getDwarfFrameSection() const {
-      return DwarfFrameSection;
-    }
-    const char *getDwarfPubNamesSection() const {
-      return DwarfPubNamesSection;
-    }
-    const char *getDwarfPubTypesSection() const {
-      return DwarfPubTypesSection;
-    }
-    const char *getDwarfStrSection() const {
-      return DwarfStrSection;
-    }
-    const char *getDwarfLocSection() const {
-      return DwarfLocSection;
-    }
-    const char *getDwarfARangesSection() const {
-      return DwarfARangesSection;
-    }
-    const char *getDwarfRangesSection() const {
-      return DwarfRangesSection;
-    }
-    const char *getDwarfMacInfoSection() const {
-      return DwarfMacInfoSection;
-    }
-    const char *getDwarfEHFrameSection() const {
-      return DwarfEHFrameSection;
-    }
-    const char *getDwarfExceptionSection() const {
-      return DwarfExceptionSection;
     }
     const char *const *getAsmCBE() const {
       return AsmTransCBE;
@@ -680,4 +525,3 @@ namespace llvm {
 }
 
 #endif
-
