@@ -15,22 +15,13 @@
 #include "llvm/Analysis/CallGraph.h"
 #include "llvm/Module.h"
 #include "llvm/Instructions.h"
+#include "llvm/IntrinsicInst.h"
 #include "llvm/Support/CallSite.h"
 #include "llvm/Support/Compiler.h"
 #include "llvm/Support/Streams.h"
 #include <ostream>
 using namespace llvm;
 
-/// isOnlyADirectCall - Return true if this callsite is *just* a direct call to
-/// the specified function.  Specifically return false if the callsite also
-/// takes the address of the function.
-static bool isOnlyADirectCall(Function *F, CallSite CS) {
-  if (!CS.getInstruction()) return false;
-  for (CallSite::arg_iterator I = CS.arg_begin(), E = CS.arg_end(); I != E; ++I)
-    if (*I == F) return false;
-  return true;
-}
-
 namespace {
 
 //===----------------------------------------------------------------------===//
@@ -52,7 +43,7 @@ class VISIBILITY_HIDDEN BasicCallGraph : public CallGraph, public ModulePass {
 
 public:
   static char ID; // Class identification, replacement for typeinfo
-  BasicCallGraph() : ModulePass((intptr_t)&ID), Root(0), 
+  BasicCallGraph() : ModulePass(&ID), Root(0), 
     ExternalCallingNode(0), CallsExternalNode(0) {}
 
   // runOnModule - Compute the call graph for the specified module.
@@ -84,7 +75,7 @@ public:
   virtual void print(std::ostream &o, const Module *M) const {
     o << "CallGraph Root is: ";
     if (Function *F = getRoot()->getFunction())
-      o << F->getName() << "\n";
+      o << F->getNameStr() << "\n";
     else
       o << "<<null function: 0x" << getRoot() << ">>\n";
     
@@ -122,7 +113,7 @@ private:
     CallGraphNode *Node = getOrInsertFunction(F);
 
     // If this function has external linkage, anything could call it.
-    if (!F->hasInternalLinkage()) {
+    if (!F->hasLocalLinkage()) {
       ExternalCallingNode->addCalledFunction(CallSite(), Node);
 
       // Found the entry point?
@@ -134,49 +125,32 @@ private:
       }
     }
 
+    // Loop over all of the users of the function, looking for non-call uses.
+    for (Value::use_iterator I = F->use_begin(), E = F->use_end(); I != E; ++I)
+      if ((!isa<CallInst>(I) && !isa<InvokeInst>(I))
+          || !CallSite(cast<Instruction>(I)).isCallee(I)) {
+        // Not a call, or being used as a parameter rather than as the callee.
+        ExternalCallingNode->addCalledFunction(CallSite(), Node);
+        break;
+      }
+
     // If this function is not defined in this translation unit, it could call
     // anything.
     if (F->isDeclaration() && !F->isIntrinsic())
       Node->addCalledFunction(CallSite(), CallsExternalNode);
 
-    // Loop over all of the users of the function... looking for callers...
-    //
-    bool isUsedExternally = false;
-    for (Value::use_iterator I = F->use_begin(), E = F->use_end(); I != E; ++I){
-      if (Instruction *Inst = dyn_cast<Instruction>(*I)) {
-        CallSite CS = CallSite::get(Inst);
-        if (isOnlyADirectCall(F, CS))
-          getOrInsertFunction(Inst->getParent()->getParent())
-              ->addCalledFunction(CS, Node);
-        else
-          isUsedExternally = true;
-      } else if (GlobalValue *GV = dyn_cast<GlobalValue>(*I)) {
-        for (Value::use_iterator I = GV->use_begin(), E = GV->use_end();
-             I != E; ++I)
-          if (Instruction *Inst = dyn_cast<Instruction>(*I)) {
-            CallSite CS = CallSite::get(Inst);
-            if (isOnlyADirectCall(F, CS))
-              getOrInsertFunction(Inst->getParent()->getParent())
-                ->addCalledFunction(CS, Node);
-            else
-              isUsedExternally = true;
-          } else {
-            isUsedExternally = true;
-          }
-      } else {                        // Can't classify the user!
-        isUsedExternally = true;
-      }
-    }
-    if (isUsedExternally)
-      ExternalCallingNode->addCalledFunction(CallSite(), Node);
-
-    // Look for an indirect function call.
+    // Look for calls by this function.
     for (Function::iterator BB = F->begin(), BBE = F->end(); BB != BBE; ++BB)
       for (BasicBlock::iterator II = BB->begin(), IE = BB->end();
            II != IE; ++II) {
-      CallSite CS = CallSite::get(II);
-      if (CS.getInstruction() && !CS.getCalledFunction())
-        Node->addCalledFunction(CS, CallsExternalNode);
+        CallSite CS = CallSite::get(II);
+        if (CS.getInstruction() && !isa<DbgInfoIntrinsic>(II)) {
+          const Function *Callee = CS.getCalledFunction();
+          if (Callee)
+            Node->addCalledFunction(CS, getOrInsertFunction(Callee));
+          else
+            Node->addCalledFunction(CS, CallsExternalNode);
+        }
       }
   }
 
@@ -190,12 +164,13 @@ private:
   }
 };
 
-RegisterAnalysisGroup<CallGraph> X("Call Graph");
-RegisterPass<BasicCallGraph> Y("basiccg", "Basic CallGraph Construction", false, true);
-RegisterAnalysisGroup<CallGraph, true> Z(Y);
-
 } //End anonymous namespace
 
+static RegisterAnalysisGroup<CallGraph> X("Call Graph");
+static RegisterPass<BasicCallGraph>
+Y("basiccg", "Basic CallGraph Construction", false, true);
+static RegisterAnalysisGroup<CallGraph, true> Z(Y);
+
 char CallGraph::ID = 0;
 char BasicCallGraph::ID = 0;
 
@@ -269,13 +244,13 @@ CallGraphNode *CallGraph::getOrInsertFunction(const Function *F) {
 
 void CallGraphNode::print(std::ostream &OS) const {
   if (Function *F = getFunction())
-    OS << "Call graph node for function: '" << F->getName() <<"'\n";
+    OS << "Call graph node for function: '" << F->getNameStr() <<"'\n";
   else
     OS << "Call graph node <<null function: 0x" << this << ">>:\n";
 
   for (const_iterator I = begin(), E = end(); I != E; ++I)
-    if (I->second->getFunction())
-      OS << "  Calls function '" << I->second->getFunction()->getName() <<"'\n";
+    if (Function *FI = I->second->getFunction())
+      OS << "  Calls function '" << FI->getNameStr() <<"'\n";
   else
     OS << "  Calls external node\n";
   OS << "\n";
@@ -283,24 +258,14 @@ void CallGraphNode::print(std::ostream &OS) const {
 
 void CallGraphNode::dump() const { print(cerr); }
 
-void CallGraphNode::removeCallEdgeTo(CallGraphNode *Callee) {
-  for (unsigned i = CalledFunctions.size(); ; --i) {
-    assert(i && "Cannot find callee to remove!");
-    if (CalledFunctions[i-1].second == Callee) {
-      CalledFunctions.erase(CalledFunctions.begin()+i-1);
-      return;
-    }
-  }
-}
-
 /// removeCallEdgeFor - This method removes the edge in the node for the
 /// specified call site.  Note that this method takes linear time, so it
 /// should be used sparingly.
 void CallGraphNode::removeCallEdgeFor(CallSite CS) {
-  for (unsigned i = CalledFunctions.size(); ; --i) {
-    assert(i && "Cannot find callee to remove!");
-    if (CalledFunctions[i-1].first == CS) {
-      CalledFunctions.erase(CalledFunctions.begin()+i-1);
+  for (CalledFunctionsVector::iterator I = CalledFunctions.begin(); ; ++I) {
+    assert(I != CalledFunctions.end() && "Cannot find callsite to remove!");
+    if (I->first == CS) {
+      CalledFunctions.erase(I);
       return;
     }
   }
@@ -319,5 +284,31 @@ void CallGraphNode::removeAnyCallEdgeTo(CallGraphNode *Callee) {
     }
 }
 
+/// removeOneAbstractEdgeTo - Remove one edge associated with a null callsite
+/// from this node to the specified callee function.
+void CallGraphNode::removeOneAbstractEdgeTo(CallGraphNode *Callee) {
+  for (CalledFunctionsVector::iterator I = CalledFunctions.begin(); ; ++I) {
+    assert(I != CalledFunctions.end() && "Cannot find callee to remove!");
+    CallRecord &CR = *I;
+    if (CR.second == Callee && !CR.first.getInstruction()) {
+      CalledFunctions.erase(I);
+      return;
+    }
+  }
+}
+
+/// replaceCallSite - Make the edge in the node for Old CallSite be for
+/// New CallSite instead.  Note that this method takes linear time, so it
+/// should be used sparingly.
+void CallGraphNode::replaceCallSite(CallSite Old, CallSite New) {
+  for (CalledFunctionsVector::iterator I = CalledFunctions.begin(); ; ++I) {
+    assert(I != CalledFunctions.end() && "Cannot find callsite to replace!");
+    if (I->first == Old) {
+      I->first = New;
+      return;
+    }
+  }
+}
+
 // Enuse that users of CallGraph.h also link with this file
 DEFINING_FILE_FOR(CallGraph)
