@@ -22,7 +22,7 @@ using namespace llvm;
 Deserializer::Deserializer(BitstreamReader& stream)
   : Stream(stream), RecIdx(0), FreeList(NULL), AbbrevNo(0), RecordCode(0) {
 
-    StreamStart = Stream.GetCurrentBitNo();
+  StreamStart = Stream.GetCurrentBitNo();
 }
 
 Deserializer::~Deserializer() {
@@ -85,7 +85,7 @@ bool Deserializer::AdvanceStream() {
         
       case bitc::END_BLOCK: {
         bool x = Stream.ReadBlockEnd();
-        assert (!x && "Error at block end.");
+        assert(!x && "Error at block end."); x=x;
         BlockStack.pop_back();
         continue;
       }
