@@ -21,15 +21,16 @@
 #include <cstring>
 using namespace llvm;
 
+namespace {
+
 static struct RegisterInterp {
   RegisterInterp() { Interpreter::Register(); }
 } InterpRegistrator;
 
-namespace llvm {
-  void LinkInInterpreter() {
-  }
 }
 
+extern "C" void LLVMLinkInInterpreter() { }
+
 /// create - Create a new interpreter object.  This can never fail.
 ///
 ExecutionEngine *Interpreter::create(ModuleProvider *MP, std::string* ErrStr) {
@@ -96,4 +97,3 @@ Interpreter::runFunction(Function *F,
 
   return ExitValue;
 }
-
