@@ -19,9 +19,11 @@
 #include "llvm/Config/config.h"
 #include "llvm/ADT/OwningPtr.h"
 #include "llvm/Support/CommandLine.h"
+#include "llvm/Support/ErrorHandling.h"
 #include "llvm/Support/MemoryBuffer.h"
 #include "llvm/Support/ManagedStatic.h"
 #include "llvm/Support/Streams.h"
+#include "llvm/Target/TargetRegistry.h"
 #include "llvm/System/Path.h"
 #include <algorithm>
 #include <functional>
@@ -45,10 +47,12 @@ TEMPLATE_INSTANTIATION(class basic_parser<unsigned>);
 TEMPLATE_INSTANTIATION(class basic_parser<double>);
 TEMPLATE_INSTANTIATION(class basic_parser<float>);
 TEMPLATE_INSTANTIATION(class basic_parser<std::string>);
+TEMPLATE_INSTANTIATION(class basic_parser<char>);
 
 TEMPLATE_INSTANTIATION(class opt<unsigned>);
 TEMPLATE_INSTANTIATION(class opt<int>);
 TEMPLATE_INSTANTIATION(class opt<std::string>);
+TEMPLATE_INSTANTIATION(class opt<char>);
 TEMPLATE_INSTANTIATION(class opt<bool>);
 
 void Option::anchor() {}
@@ -60,6 +64,7 @@ void parser<unsigned>::anchor() {}
 void parser<double>::anchor() {}
 void parser<float>::anchor() {}
 void parser<std::string>::anchor() {}
+void parser<char>::anchor() {}
 
 //===----------------------------------------------------------------------===//
 
@@ -120,7 +125,7 @@ static void GetOptionInfo(std::vector<Option*> &PositionalOpts,
       if (!OptionsMap.insert(std::pair<std::string,Option*>(OptionNames[i],
                                                             O)).second) {
         cerr << ProgramName << ": CommandLine Error: Argument '"
-             << OptionNames[0] << "' defined more than once!\n";
+             << OptionNames[i] << "' defined more than once!\n";
       }
     }
 
@@ -172,6 +177,9 @@ static Option *LookupOption(const char *&Arg, const char *&Value,
 static inline bool ProvideOption(Option *Handler, const char *ArgName,
                                  const char *Value, int argc, char **argv,
                                  int &i) {
+  // Is this a multi-argument option?
+  unsigned NumAdditionalVals = Handler->getNumAdditionalVals();
+
   // Enforce value requirements
   switch (Handler->getValueExpectedFlag()) {
   case ValueRequired:
@@ -179,13 +187,17 @@ static inline bool ProvideOption(Option *Handler, const char *ArgName,
       if (i+1 < argc) {     // Steal the next argument, like for '-o filename'
         Value = argv[++i];
       } else {
-        return Handler->error(" requires a value!");
+        return Handler->error("requires a value!");
       }
     }
     break;
   case ValueDisallowed:
+    if (NumAdditionalVals > 0)
+      return Handler->error("multi-valued option specified"
+      " with ValueDisallowed modifier!");
+
     if (Value)
-      return Handler->error(" does not allow a value! '" +
+      return Handler->error("does not allow a value! '" +
                             std::string(Value) + "' specified.");
     break;
   case ValueOptional:
@@ -194,12 +206,38 @@ static inline bool ProvideOption(Option *Handler, const char *ArgName,
     cerr << ProgramName
          << ": Bad ValueMask flag! CommandLine usage error:"
          << Handler->getValueExpectedFlag() << "\n";
-    abort();
-    break;
+    llvm_unreachable(0);
   }
 
-  // Run the handler now!
-  return Handler->addOccurrence(i, ArgName, Value ? Value : "");
+  // If this isn't a multi-arg option, just run the handler.
+  if (NumAdditionalVals == 0) {
+    return Handler->addOccurrence(i, ArgName, Value ? Value : "");
+  }
+  // If it is, run the handle several times.
+  else {
+    bool MultiArg = false;
+
+    if (Value) {
+      if (Handler->addOccurrence(i, ArgName, Value, MultiArg))
+        return true;
+      --NumAdditionalVals;
+      MultiArg = true;
+    }
+
+    while (NumAdditionalVals > 0) {
+
+      if (i+1 < argc) {
+        Value = argv[++i];
+      } else {
+        return Handler->error("not enough values!");
+      }
+      if (Handler->addOccurrence(i, ArgName, Value, MultiArg))
+        return true;
+      MultiArg = true;
+      --NumAdditionalVals;
+    }
+    return false;
+  }
 }
 
 static bool ProvidePositionalOption(Option *Handler, const std::string &Arg,
@@ -354,23 +392,27 @@ static void ExpandResponseFiles(int argc, char** argv,
       // Check that the response file is not empty (mmap'ing empty
       // files can be problematic).
       const sys::FileStatus *FileStat = respFile.getFileStatus();
-      if (!FileStat)
-        continue;
-      if (FileStat->getSize() == 0)
-        continue;
+      if (FileStat && FileStat->getSize() != 0) {
 
-      // Mmap the response file into memory.
-      OwningPtr<MemoryBuffer>
-        respFilePtr(MemoryBuffer::getFile(respFile.c_str()));
+        // Mmap the response file into memory.
+        OwningPtr<MemoryBuffer>
+          respFilePtr(MemoryBuffer::getFile(respFile.c_str()));
 
-      if (respFilePtr == 0)
-        continue;
+        // If we could open the file, parse its contents, otherwise
+        // pass the @file option verbatim.
 
-      ParseCStringVector(newArgv, respFilePtr->getBufferStart());
-    }
-    else {
-      newArgv.push_back(strdup(arg));
+        // TODO: we should also support recursive loading of response files,
+        // since this is how gcc behaves. (From their man page: "The file may
+        // itself contain additional @file options; any such options will be
+        // processed recursively.")
+
+        if (respFilePtr != 0) {
+          ParseCStringVector(newArgv, respFilePtr->getBufferStart());
+          continue;
+        }
+      }
     }
+    newArgv.push_back(strdup(arg));
   }
 }
 
@@ -394,8 +436,6 @@ void cl::ParseCommandLineOptions(int argc, char **argv,
     argc = static_cast<int>(newArgv.size());
   }
 
-  sys::Path progname(argv[0]);
-
   // Copy the program name into ProgName, making sure not to overflow it.
   std::string ProgName = sys::Path(argv[0]).getLast();
   if (ProgName.size() > 79) ProgName.resize(79);
@@ -430,7 +470,7 @@ void cl::ParseCommandLineOptions(int argc, char **argv,
         // unless there is only one positional argument...
         if (PositionalOpts.size() > 2)
           ErrorParsing |=
-            Opt->error(" error - this positional option will never be matched, "
+            Opt->error("error - this positional option will never be matched, "
                        "because it does not Require a value, and a "
                        "cl::ConsumeAfter option is active!");
       } else if (UnboundedFound && !Opt->ArgStr[0]) {
@@ -438,7 +478,7 @@ void cl::ParseCommandLineOptions(int argc, char **argv,
         // not specified after an option that eats all extra arguments, or this
         // one will never get any!
         //
-        ErrorParsing |= Opt->error(" error - option can never match, because "
+        ErrorParsing |= Opt->error("error - option can never match, because "
                                    "another positional argument will match an "
                                    "unbounded number of values, and this option"
                                    " does not require a value!");
@@ -654,7 +694,7 @@ void cl::ParseCommandLineOptions(int argc, char **argv,
           ValNo++;
           break;
         default:
-          assert(0 && "Internal error, unexpected NumOccurrences flag in "
+          llvm_unreachable("Internal error, unexpected NumOccurrences flag in "
                  "positional argument processing!");
         }
       }
@@ -697,7 +737,7 @@ void cl::ParseCommandLineOptions(int argc, char **argv,
     case Required:
     case OneOrMore:
       if (I->second->getNumOccurrences() == 0) {
-        I->second->error(" must be specified at least once!");
+        I->second->error("must be specified at least once!");
         ErrorParsing = true;
       }
       // Fall through
@@ -740,22 +780,24 @@ bool Option::error(std::string Message, const char *ArgName) {
 }
 
 bool Option::addOccurrence(unsigned pos, const char *ArgName,
-                           const std::string &Value) {
-  NumOccurrences++;   // Increment the number of times we have been seen
+                           const std::string &Value,
+                           bool MultiArg) {
+  if (!MultiArg)
+    NumOccurrences++;   // Increment the number of times we have been seen
 
   switch (getNumOccurrencesFlag()) {
   case Optional:
     if (NumOccurrences > 1)
-      return error(": may only occur zero or one times!", ArgName);
+      return error("may only occur zero or one times!", ArgName);
     break;
   case Required:
     if (NumOccurrences > 1)
-      return error(": must occur exactly one time!", ArgName);
+      return error("must occur exactly one time!", ArgName);
     // Fall through
   case OneOrMore:
   case ZeroOrMore:
   case ConsumeAfter: break;
-  default: return error(": bad num occurrences flag value!");
+  default: return error("bad num occurrences flag value!");
   }
 
   return handleOccurrence(pos, ArgName, Value);
@@ -782,8 +824,8 @@ size_t alias::getOptionWidth() const {
 // Print out the option for the alias.
 void alias::printOptionInfo(size_t GlobalWidth) const {
   size_t L = std::strlen(ArgStr);
-  cout << "  -" << ArgStr << std::string(GlobalWidth-L-6, ' ') << " - "
-       << HelpStr << "\n";
+  cerr << "  -" << ArgStr << std::string(GlobalWidth-L-6, ' ') << " - "
+         << HelpStr << "\n";
 }
 
 
@@ -831,7 +873,7 @@ bool parser<bool>::parse(Option &O, const char *ArgName,
   } else if (Arg == "false" || Arg == "FALSE" || Arg == "False" || Arg == "0") {
     Value = false;
   } else {
-    return O.error(": '" + Arg +
+    return O.error("'" + Arg +
                    "' is invalid value for boolean argument! Try 0 or 1");
   }
   return false;
@@ -844,10 +886,11 @@ bool parser<boolOrDefault>::parse(Option &O, const char *ArgName,
   if (Arg == "" || Arg == "true" || Arg == "TRUE" || Arg == "True" ||
       Arg == "1") {
     Value = BOU_TRUE;
-  } else if (Arg == "false" || Arg == "FALSE" || Arg == "False" || Arg == "0") {
+  } else if (Arg == "false" || Arg == "FALSE"
+             || Arg == "False" || Arg == "0") {
     Value = BOU_FALSE;
   } else {
-    return O.error(": '" + Arg +
+    return O.error("'" + Arg +
                    "' is invalid value for boolean argument! Try 0 or 1");
   }
   return false;
@@ -860,7 +903,7 @@ bool parser<int>::parse(Option &O, const char *ArgName,
   char *End;
   Value = (int)strtol(Arg.c_str(), &End, 0);
   if (*End != 0)
-    return O.error(": '" + Arg + "' value invalid for integer argument!");
+    return O.error("'" + Arg + "' value invalid for integer argument!");
   return false;
 }
 
@@ -875,7 +918,7 @@ bool parser<unsigned>::parse(Option &O, const char *ArgName,
   if (((V == ULONG_MAX) && (errno == ERANGE))
       || (*End != 0)
       || (Value != V))
-    return O.error(": '" + Arg + "' value invalid for uint argument!");
+    return O.error("'" + Arg + "' value invalid for uint argument!");
   return false;
 }
 
@@ -886,7 +929,7 @@ static bool parseDouble(Option &O, const std::string &Arg, double &Value) {
   char *End;
   Value = strtod(ArgStart, &End);
   if (*End != 0)
-    return O.error(": '" +Arg+ "' value invalid for floating point argument!");
+    return O.error("'" + Arg + "' value invalid for floating point argument!");
   return false;
 }
 
@@ -953,7 +996,7 @@ void generic_parser_base::printOptionInfo(const Option &O,
     for (unsigned i = 0, e = getNumOptions(); i != e; ++i) {
       size_t NumSpaces = GlobalWidth-strlen(getOption(i))-8;
       cout << "    =" << getOption(i) << std::string(NumSpaces, ' ')
-           << " - " << getDescription(i) << "\n";
+           << " -   " << getDescription(i) << "\n";
     }
   } else {
     if (O.HelpStr[0])
@@ -1097,6 +1140,27 @@ public:
         cout << " with assertions";
 #endif
         cout << ".\n";
+        cout << "  Built " << __DATE__ << "(" << __TIME__ << ").\n";
+        cout << "\n";
+        cout << "  Registered Targets:\n";
+
+        std::vector<std::pair<std::string, const Target*> > Targets;
+        size_t Width = 0;
+        for (TargetRegistry::iterator it = TargetRegistry::begin(), 
+               ie = TargetRegistry::end(); it != ie; ++it) {
+          Targets.push_back(std::make_pair(it->getName(), &*it));
+          Width = std::max(Width, ::strlen(it->getName()));
+        }
+        std::sort(Targets.begin(), Targets.end());
+
+        for (unsigned i = 0, e = Targets.size(); i != e; ++i) {
+          const Target *T = Targets[i].second;
+          cout << "    " << T->getName()
+               << std::string(Width - ::strlen(T->getName()), ' ') << " - "
+               << T->getShortDescription() << "\n";
+        }
+        if (Targets.empty())
+          cout << "    (none)\n";
   }
   void operator=(bool OptionWasSpecified) {
     if (OptionWasSpecified) {
