@@ -11,27 +11,20 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef ARMTARGETASMINFO_H
-#define ARMTARGETASMINFO_H
+#ifndef LLVM_ARMTARGETASMINFO_H
+#define LLVM_ARMTARGETASMINFO_H
 
-#include "llvm/Target/TargetAsmInfo.h"
-#include "ARMSubtarget.h"
+#include "llvm/Target/DarwinTargetAsmInfo.h"
 
 namespace llvm {
 
-  // Forward declaration.
-  class ARMTargetMachine;
-
-  struct ARMTargetAsmInfo : public TargetAsmInfo {
-    explicit ARMTargetAsmInfo(const ARMTargetMachine &TM);
-
-    const ARMSubtarget *Subtarget;
-
-    virtual unsigned getInlineAsmLength(const char *Str) const;
-    unsigned countArguments(const char *p) const;
-    unsigned countString(const char *p) const;
+  struct ARMDarwinTargetAsmInfo : public DarwinTargetAsmInfo {
+    explicit ARMDarwinTargetAsmInfo();
   };
 
+  struct ARMELFTargetAsmInfo : public TargetAsmInfo {
+    explicit ARMELFTargetAsmInfo();
+  };
 
 } // namespace llvm
 
