@@ -11,20 +11,18 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef PPCTARGETASMINFO_H
-#define PPCTARGETASMINFO_H
+#ifndef SPUTARGETASMINFO_H
+#define SPUTARGETASMINFO_H
 
 #include "llvm/Target/TargetAsmInfo.h"
 
 namespace llvm {
-
-  // Forward declaration.
-  class SPUTargetMachine;
-
-  struct SPUTargetAsmInfo : public TargetAsmInfo {
-    SPUTargetAsmInfo(const SPUTargetMachine &TM);
+  class Target;
+  class StringRef;
+  
+  struct SPULinuxTargetAsmInfo : public TargetAsmInfo {
+    explicit SPULinuxTargetAsmInfo(const Target &T, const StringRef &TT);
   };
-
 } // namespace llvm
 
-#endif
+#endif /* SPUTARGETASMINFO_H */
