@@ -12,26 +12,16 @@
 //===----------------------------------------------------------------------===//
 
 #include "MipsTargetAsmInfo.h"
-#include "MipsTargetMachine.h"
-
 using namespace llvm;
 
-MipsTargetAsmInfo::MipsTargetAsmInfo(const MipsTargetMachine &TM) {
-  AlignmentIsInBytes   = false;
-  Data16bitsDirective  = "\t.half\t";
-  Data32bitsDirective  = "\t.word\t";
-  PrivateGlobalPrefix  = "$";
-  JumpTableDataSection = "\t.rdata";
-  CommentString        = "#";
-  ReadOnlySection      = "\t.rdata";
-  ZeroDirective        = "\t.space\t";
-  BSSSection           = "\t.section\t.bss";
-  LCOMMDirective       = "\t.lcomm\t";
-
-  if (TM.getRelocationModel() == Reloc::Static)
-    JumpTableDirective = "\t.word\t";
-  else    
-    JumpTableDirective = "\t.gpword\t";
-
+MipsTargetAsmInfo::MipsTargetAsmInfo(const Target &T, const StringRef &TT) {
+  AlignmentIsInBytes          = false;
   COMMDirectiveTakesAlignment = true;
+  Data16bitsDirective         = "\t.half\t";
+  Data32bitsDirective         = "\t.word\t";
+  Data64bitsDirective         = NULL;
+  PrivateGlobalPrefix         = "$";
+  CommentString               = "#";
+  ZeroDirective               = "\t.space\t";
+  PICJumpTableDirective       = "\t.gpword\t";
 }
