@@ -17,12 +17,12 @@
 #include "llvm/Target/TargetAsmInfo.h"
 
 namespace llvm {
-
-  // Forward declaration.
-  class MipsTargetMachine;
-
-  struct MipsTargetAsmInfo : public TargetAsmInfo {
-    explicit MipsTargetAsmInfo(const MipsTargetMachine &TM);
+  class Target;
+  class StringRef;
+  
+  class MipsTargetAsmInfo : public TargetAsmInfo {
+  public:
+    explicit MipsTargetAsmInfo(const Target &T, const StringRef &TT);
   };
 
 } // namespace llvm
