@@ -15,26 +15,16 @@
 #define PPCTARGETASMINFO_H
 
 #include "llvm/Target/TargetAsmInfo.h"
+#include "llvm/Target/DarwinTargetAsmInfo.h"
 
 namespace llvm {
 
-  // Forward declaration.
-  class PPCTargetMachine;
-  
-  struct PPCTargetAsmInfo : public TargetAsmInfo {
-    explicit PPCTargetAsmInfo(const PPCTargetMachine &TM);
+  struct PPCDarwinTargetAsmInfo : public DarwinTargetAsmInfo {
+    explicit PPCDarwinTargetAsmInfo(bool is64Bit);
   };
 
-  struct DarwinTargetAsmInfo : public PPCTargetAsmInfo {
-    explicit DarwinTargetAsmInfo(const PPCTargetMachine &TM);
-    virtual unsigned PreferredEHDataFormat(DwarfEncoding::Target Reason,
-                                           bool Global) const;
-  };
-
-  struct LinuxTargetAsmInfo : public PPCTargetAsmInfo {
-    explicit LinuxTargetAsmInfo(const PPCTargetMachine &TM);
-    virtual unsigned PreferredEHDataFormat(DwarfEncoding::Target Reason,
-                                           bool Global) const;
+  struct PPCLinuxTargetAsmInfo : public TargetAsmInfo {
+    explicit PPCLinuxTargetAsmInfo(bool is64Bit);
   };
 
 } // namespace llvm
