@@ -15,23 +15,28 @@
 #define X86TARGETASMINFO_H
 
 #include "llvm/Target/TargetAsmInfo.h"
+#include "llvm/Target/COFFTargetAsmInfo.h"
+#include "llvm/Target/DarwinTargetAsmInfo.h"
 
 namespace llvm {
+  class Triple;
 
-  // Forward declaration.
-  class X86TargetMachine;
-
-  struct X86TargetAsmInfo : public TargetAsmInfo {
-    explicit X86TargetAsmInfo(const X86TargetMachine &TM);
-    
-    virtual bool ExpandInlineAsm(CallInst *CI) const;
-    virtual unsigned PreferredEHDataFormat(DwarfEncoding::Target Reason,
-                                           bool Global) const;
-
-  private:
-    const X86TargetMachine* X86TM;
-    bool LowerToBSwap(CallInst *CI) const;
+  struct X86DarwinTargetAsmInfo : public DarwinTargetAsmInfo {
+    explicit X86DarwinTargetAsmInfo(const Triple &Triple);
   };
+
+  struct X86ELFTargetAsmInfo : public TargetAsmInfo {
+    explicit X86ELFTargetAsmInfo(const Triple &Triple);
+  };
+
+  struct X86COFFTargetAsmInfo : public COFFTargetAsmInfo {
+    explicit X86COFFTargetAsmInfo(const Triple &Triple);
+  };
+
+  struct X86WinTargetAsmInfo : public TargetAsmInfo {
+    explicit X86WinTargetAsmInfo(const Triple &Triple);
+  };
+
 } // namespace llvm
 
 #endif
