@@ -17,12 +17,17 @@
 #ifndef BUGPOINT_TOOLRUNNER_H
 #define BUGPOINT_TOOLRUNNER_H
 
+#include "llvm/ADT/Triple.h"
+#include "llvm/Support/CommandLine.h"
 #include "llvm/Support/SystemUtils.h"
 #include <exception>
 #include <vector>
 
 namespace llvm {
 
+extern cl::opt<bool> SaveTemps;
+extern Triple TargetTriple;
+
 class CBE;
 class LLC;
 
@@ -43,14 +48,19 @@ public:
 // GCC abstraction
 //
 class GCC {
-  sys::Path GCCPath;          // The path to the gcc executable
-  sys::Path RSHPath;          // The path to the rsh executable
-  GCC(const sys::Path &gccPath, const sys::Path &rshPath)
-    : GCCPath(gccPath), RSHPath(rshPath) { }
+  sys::Path GCCPath;                // The path to the gcc executable.
+  sys::Path RemoteClientPath;       // The path to the rsh / ssh executable.
+  std::vector<std::string> gccArgs; // GCC-specific arguments.
+  GCC(const sys::Path &gccPath, const sys::Path &RemotePath,
+      const std::vector<std::string> *GCCArgs)
+    : GCCPath(gccPath), RemoteClientPath(RemotePath) {
+    if (GCCArgs) gccArgs = *GCCArgs;
+  }
 public:
   enum FileType { AsmFile, CFile };
 
-  static GCC *create(const std::string &ProgramPath, std::string &Message);
+  static GCC *create(std::string &Message,
+                     const std::vector<std::string> *Args);
 
   /// ExecuteProgram - Execute the program specified by "ProgramFile" (which is
   /// either a .s file, or a .c file, specified by FileType), with the specified
@@ -85,21 +95,20 @@ public:
 ///
 class AbstractInterpreter {
 public:
-  static CBE *createCBE(const std::string &ProgramPath, std::string &Message,
-                        const std::vector<std::string> *Args = 0);
-  static LLC *createLLC(const std::string &ProgramPath, std::string &Message,
-                        const std::vector<std::string> *Args = 0);
+  static CBE *createCBE(const char *Argv0, std::string &Message,
+                        const std::vector<std::string> *Args = 0,
+                        const std::vector<std::string> *GCCArgs = 0);
+  static LLC *createLLC(const char *Argv0, std::string &Message,
+                        const std::vector<std::string> *Args = 0,
+                        const std::vector<std::string> *GCCArgs = 0);
 
-  static AbstractInterpreter* createLLI(const std::string &ProgramPath,
-                                        std::string &Message,
+  static AbstractInterpreter* createLLI(const char *Argv0, std::string &Message,
                                         const std::vector<std::string> *Args=0);
 
-  static AbstractInterpreter* createJIT(const std::string &ProgramPath,
-                                        std::string &Message,
+  static AbstractInterpreter* createJIT(const char *Argv0, std::string &Message,
                                         const std::vector<std::string> *Args=0);
 
-  static AbstractInterpreter* createCustom(const std::string &ProgramPath,
-                                           std::string &Message,
+  static AbstractInterpreter* createCustom(std::string &Message,
                                            const std::string &ExecCommandLine);
 
 
@@ -139,14 +148,15 @@ public:
 // CBE Implementation of AbstractIntepreter interface
 //
 class CBE : public AbstractInterpreter {
-  sys::Path LLCPath;          // The path to the `llc' executable
-  std::vector<std::string> ToolArgs; // Extra args to pass to LLC
+  sys::Path LLCPath;                 // The path to the `llc' executable.
+  std::vector<std::string> ToolArgs; // Extra args to pass to LLC.
   GCC *gcc;
 public:
   CBE(const sys::Path &llcPath, GCC *Gcc,
-      const std::vector<std::string> *Args) : LLCPath(llcPath), gcc(Gcc) {
+      const std::vector<std::string> *Args)
+    : LLCPath(llcPath), gcc(Gcc) {
     ToolArgs.clear ();
-    if (Args) { ToolArgs = *Args; }
+    if (Args) ToolArgs = *Args;
   }
   ~CBE() { delete gcc; }
 
@@ -180,14 +190,18 @@ public:
 // LLC Implementation of AbstractIntepreter interface
 //
 class LLC : public AbstractInterpreter {
-  std::string LLCPath;          // The path to the LLC executable
-  std::vector<std::string> ToolArgs; // Extra args to pass to LLC
+  std::string LLCPath;               // The path to the LLC executable.
+  std::vector<std::string> ToolArgs; // Extra args to pass to LLC.
+  std::vector<std::string> gccArgs;  // Extra args to pass to GCC.
   GCC *gcc;
 public:
   LLC(const std::string &llcPath, GCC *Gcc,
-    const std::vector<std::string> *Args) : LLCPath(llcPath), gcc(Gcc) {
-    ToolArgs.clear ();
-    if (Args) { ToolArgs = *Args; }
+      const std::vector<std::string> *Args,
+      const std::vector<std::string> *GCCArgs)
+    : LLCPath(llcPath), gcc(Gcc) {
+    ToolArgs.clear();
+    if (Args) ToolArgs = *Args;
+    if (GCCArgs) gccArgs = *GCCArgs;
   }
   ~LLC() { delete gcc; }
 
