@@ -15,17 +15,18 @@
 //
 //===----------------------------------------------------------------------===//
 
+#include "llvm/LLVMContext.h"
 #include "llvm/Module.h"
 #include "llvm/Assembly/Parser.h"
 #include "llvm/Analysis/Verifier.h"
 #include "llvm/Bitcode/ReaderWriter.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/ManagedStatic.h"
-#include "llvm/Support/Streams.h"
+#include "llvm/Support/PrettyStackTrace.h"
+#include "llvm/Support/SourceMgr.h"
 #include "llvm/Support/SystemUtils.h"
+#include "llvm/Support/raw_ostream.h"
 #include "llvm/System/Signals.h"
-#include <fstream>
-#include <iostream>
 #include <memory>
 using namespace llvm;
 
@@ -50,52 +51,56 @@ DisableVerify("disable-verify", cl::Hidden,
               cl::desc("Do not run verifier on input LLVM (dangerous!)"));
 
 int main(int argc, char **argv) {
-  llvm_shutdown_obj X;  // Call llvm_shutdown() on exit.
-  cl::ParseCommandLineOptions(argc, argv, "llvm .ll -> .bc assembler\n");
+  // Print a stack trace if we signal out.
   sys::PrintStackTraceOnErrorSignal();
+  PrettyStackTraceProgram X(argc, argv);
+  LLVMContext &Context = getGlobalContext();
+  llvm_shutdown_obj Y;  // Call llvm_shutdown() on exit.
+  cl::ParseCommandLineOptions(argc, argv, "llvm .ll -> .bc assembler\n");
 
   int exitCode = 0;
-  std::ostream *Out = 0;
+  raw_ostream *Out = 0;
   try {
     // Parse the file now...
-    ParseError Err;
-    std::auto_ptr<Module> M(ParseAssemblyFile(InputFilename,&Err));
+    SMDiagnostic Err;
+    std::auto_ptr<Module> M(ParseAssemblyFile(InputFilename, Err, Context));
     if (M.get() == 0) {
-      cerr << argv[0] << ": " << Err.getMessage() << "\n"; 
+      Err.Print(argv[0], errs());
       return 1;
     }
 
     if (!DisableVerify) {
       std::string Err;
       if (verifyModule(*M.get(), ReturnStatusAction, &Err)) {
-        cerr << argv[0]
-             << ": assembly parsed, but does not verify as correct!\n";
-        cerr << Err;
+        errs() << argv[0]
+               << ": assembly parsed, but does not verify as correct!\n";
+        errs() << Err;
         return 1;
       } 
     }
 
-    if (DumpAsm) cerr << "Here's the assembly:\n" << *M.get();
+    if (DumpAsm) errs() << "Here's the assembly:\n" << *M.get();
 
     if (OutputFilename != "") {   // Specified an output filename?
       if (OutputFilename != "-") {  // Not stdout?
-        if (!Force && std::ifstream(OutputFilename.c_str())) {
-          // If force is not specified, make sure not to overwrite a file!
-          cerr << argv[0] << ": error opening '" << OutputFilename
-               << "': file exists!\n"
-               << "Use -f command line argument to force output\n";
+        std::string ErrorInfo;
+        Out = new raw_fd_ostream(OutputFilename.c_str(), /*Binary=*/true,
+                                 Force, ErrorInfo);
+        if (!ErrorInfo.empty()) {
+          errs() << ErrorInfo << '\n';
+          if (!Force)
+            errs() << "Use -f command line argument to force output\n";
+          delete Out;
           return 1;
         }
-        Out = new std::ofstream(OutputFilename.c_str(), std::ios::out |
-                                std::ios::trunc | std::ios::binary);
       } else {                      // Specified stdout
-        // FIXME: cout is not binary!
-        Out = &std::cout;
+        // FIXME: outs() is not binary!
+        Out = &outs();
       }
     } else {
       if (InputFilename == "-") {
         OutputFilename = "-";
-        Out = &std::cout;
+        Out = &outs();
       } else {
         std::string IFN = InputFilename;
         int Len = IFN.length();
@@ -107,39 +112,34 @@ int main(int argc, char **argv) {
         }
         OutputFilename += ".bc";
 
-        if (!Force && std::ifstream(OutputFilename.c_str())) {
-          // If force is not specified, make sure not to overwrite a file!
-          cerr << argv[0] << ": error opening '" << OutputFilename
-               << "': file exists!\n"
-               << "Use -f command line argument to force output\n";
+        std::string ErrorInfo;
+        Out = new raw_fd_ostream(OutputFilename.c_str(), /*Binary=*/true,
+                                 Force, ErrorInfo);
+        if (!ErrorInfo.empty()) {
+          errs() << ErrorInfo << '\n';
+          if (!Force)
+            errs() << "Use -f command line argument to force output\n";
+          delete Out;
           return 1;
         }
-
-        Out = new std::ofstream(OutputFilename.c_str(), std::ios::out |
-                                std::ios::trunc | std::ios::binary);
         // Make sure that the Out file gets unlinked from the disk if we get a
         // SIGINT
         sys::RemoveFileOnSignal(sys::Path(OutputFilename));
       }
     }
 
-    if (!Out->good()) {
-      cerr << argv[0] << ": error opening " << OutputFilename << "!\n";
-      return 1;
-    }
-
     if (!DisableOutput)
       if (Force || !CheckBitcodeOutputToConsole(Out,true))
         WriteBitcodeToFile(M.get(), *Out);
   } catch (const std::string& msg) {
-    cerr << argv[0] << ": " << msg << "\n";
+    errs() << argv[0] << ": " << msg << "\n";
     exitCode = 1;
   } catch (...) {
-    cerr << argv[0] << ": Unexpected unknown exception occurred.\n";
+    errs() << argv[0] << ": Unexpected unknown exception occurred.\n";
     exitCode = 1;
   }
 
-  if (Out != &std::cout) delete Out;
+  if (Out != &outs()) delete Out;
   return exitCode;
 }
 
