@@ -21,6 +21,7 @@
 #include "llvm/Support/FileUtilities.h"
 #include "llvm/ADT/StringExtras.h"
 #include <iostream>
+#include <cstdlib>
 using namespace llvm;
 
 /// getCurrentLanguage - Return the current source language that the user is
@@ -63,7 +64,7 @@ void CLIDebugger::startProgramRunning() {
     TheProgramInfo = 0;
     CurrentFile = 0;
 
-    Dbg.loadProgram(Program.toString());
+    Dbg.loadProgram(Program.toString(), Context);
     TheProgramInfo = new ProgramInfo(Dbg.getProgram());
   }
 
@@ -243,7 +244,7 @@ void CLIDebugger::fileCommand(std::string &Options) {
     std::cout << "Unloaded program.\n";
   } else {
     std::cout << "Loading program... " << std::flush;
-    Dbg.loadProgram(Prog);
+    Dbg.loadProgram(Prog, Context);
     assert(Dbg.isProgramLoaded() &&
            "loadProgram succeeded, but not program loaded!");
     TheProgramInfo = new ProgramInfo(Dbg.getProgram());
@@ -448,11 +449,12 @@ void CLIDebugger::downCommand(std::string &Options) {
   unsigned CurFrame = RI.getCurrentFrameIdx();
 
   // Check to see if we can go up the specified number of frames.
-  if (CurFrame < Num)
+  if (CurFrame < Num) {
     if (Num == 1)
       throw "Bottom (i.e., innermost) frame selected; you cannot go down.";
     else
       throw "Cannot go down " + utostr(Num) + " frames!";
+  }
 
   RI.setCurrentFrameIdx(CurFrame-Num);
   printProgramLocation();
@@ -498,7 +500,7 @@ void CLIDebugger::breakCommand(std::string &Options) {
   if (File == 0)
     throw "Unknown file to place breakpoint!";
 
-  std::cerr << "Break: " << File->getFilename() << ":" << LineNo << "\n";
+  errs() << "Break: " << File->getFilename() << ":" << LineNo << "\n";
 
   throw "breakpoints not implemented yet!";
 }
